import wx

class 图片组类(wx.ImageList):
    _宽度 = 64
    _高度 = 64
    def 加入图片(self, 文件路径, 宽度=0, 高度=0):
        img = wx.Image(文件路径, wx.BITMAP_TYPE_ANY)
        if 宽度 == 0:
            img = img.Scale(self._宽度, self._高度)
        else:
            img = img.Scale(宽度, 高度)

        img = img.ConvertToBitmap()
        return super().Add(img)

    def 加入(self, *__args):
        return super().Add(*__args)

    def 创建(self, width, height, mask=True, initialCount=1):
        return super().Create(width, height, mask, initialCount)

    def 绘制(self, index, dc, x, y, flags=None, solidBackground=False):
        return super().Draw(index, dc, x, y, flags, solidBackground)

    def 取位图(self, index):
        return super().GetBitmap(index)

    def 取Icon(self, index):
        return super().GetIcon(index)

    def 取图片数量(self):
        return super().GetImageCount(self)

    def 取大小(self, index=None):
        return super().GetSize(index)

    def 删除(self, index):
        return super().Remove(index)

    def 删除所有(self):
        return super().RemoveAll(self)

    def 替换(self, index, *__args):
        return super().Replace(index, *__args)

    def __init__(self, width=None, height=None, mask=True, initialCount=1):
        self._宽度 = width
        self._高度 = height
        return super().__init__(width, height, mask, initialCount)



"""

.. Hint::
    gevent实现的协程池

    调用时需要引入猴子补丁 需要安装gevent

    pip install gevent

    代码中需要引入猴子补丁

    import gevent
    from gevent import monkey
    monkey.patch_all()

    还有另一个版本的协程池实现 协程池 为python原生支持的实现

.. literalinclude:: ../../../pyefun/协程池/协程池_test.py
    :language: python
    :caption: 代码示例
    :linenos:

"""
import gevent
from gevent import monkey
monkey.patch_all()
from gevent.threadpool import ThreadPool

from pyefun import 事件锁

class 协程池():

    def __init__(self, 协程数量, 投递任务时阻塞=True):
        self.pool = ThreadPool(协程数量)
        self.投递任务时阻塞 = 投递任务时阻塞
        if (投递任务时阻塞 == True):
            self.已投递任务数量 = 0
            self.最大线程数量 = 协程数量
            self.锁 = 事件锁()

    def 等待协程完成任务(self, 任务列表):
        gevent.joinall(任务列表)

    def 投递任务(self, 任务函数, *args, **kwargs):
        if self.投递任务时阻塞:
            if (self.已投递任务数量 >= self.最大线程数量):
                self.锁.堵塞()
                self.锁.等待()
            self.已投递任务数量 = self.已投递任务数量 + 1

        job = self.pool.spawn(任务函数, *args, **kwargs)
        if self.投递任务时阻塞:
            def 回调函数x(e):
                self.已投递任务数量 = self.已投递任务数量 - 1
                self.锁.通行()
            job.rawlink(回调函数x)
        return job

    def 设置任务结束回调函数(self, future, 回调函数):
        """
        投递任务返回的对象
        def 回调函数(线程返回的结果):
            print("当前线程", current_thread().name)
            print("线程返回的结果", future.result())
        """

        def 匿名函数(future):
            回调函数(future.get())

        future.rawlink(匿名函数)

    def 等待(self):
        """等待所有任务完成"""
        self.pool.join()
        # gevent.wait()

    def 关闭(self):
        """等待所有任务完成"""
        self.pool.kill()



"""

.. Hint::
    配置项 读取ini配置文件

.. literalinclude:: ../../../pyefun/配置项_test.py
    :language: python
    :caption: 代码示例
    :linenos:
    :lines: 1-100

"""

import configparser
import io

class 配置项(configparser.ConfigParser):
    def __init__(self, 内容=""):
        pass
        super().__init__()
        if 内容:
            self.加载(内容)

    def 加载(self, 内容):
        self.read_string(内容)  # python3
        return self

    def 取所有项名(self, 节名):
        return self.options(节名)

    def 取所有项名和值(self, 节名):
        return self.items(节名)

    def 取所有节名(self):
        return self.sections()

    def 读配置项(self, 节名称, 配置项名称, 默认值=""):
        try:
            return self.get(节名称, 配置项名称)
        except:
            return 默认值

    def 写配置项(self, 节名称, 配置项名称, 值):
        if not self.has_section(节名称):
            self.add_section(节名称)
        return self.set(节名称, 配置项名称, 值)

    def 删除配置项(self, 节名称, 配置项名称):
        return self.remove_option(节名称, 配置项名称)

    def 删除节名(self, 节名称):
        return self.remove_section(节名称)

    def 取数据(self):
        fp = io.StringIO()
        self.write(fp)
        text = fp.getvalue()
        fp.close()
        return text

    def 取所有结果(self):
        字典 = dict()
        所有节名 = self.取所有节名()
        for 节名 in 所有节名:
            字典[节名]  = dict()
            for v in self.取所有项名和值(节名):
                字典[节名][v[0]] = v[1]
        return 字典



"""
json编码解码飞快的库


https://github.com/ijl/orjson
还有
https://github.com/ultrajson/ultrajson
"""

import ujson


def json到文本(data, html编码=True, ascii编码=False, 斜杠转义=True, 缩进=0):
    return ujson.dumps(data,
                       encode_html_chars=html编码,
                       ensure_ascii=ascii编码,
                       escape_forward_slashes=斜杠转义,
                       indent=缩进)


def json加载(data):
    return ujson.loads(data)



import unittest

from pyefun import *
from pyefun.环境变量 import *
from fc import *

class TestFC(unittest.TestCase):
    def test_4(self):
        pass
        print("111111111111111111111111")

    def test_1(self):
        # 调用http函数
        环境变量_从文本中加载至系统(读入文本(取运行目录() + "/.env"))
        access_key_id = 取环境变量("access_key_id")
        access_key_secret = 取环境变量("access_key_secret")
        endpoint = 取环境变量("endpoint")
        """
        access_key_id=
        access_key_secret=
        #endpoint=.cn-beijing.fc.aliyuncs.com
        endpoint=.cn-hangzhou.fc.aliyuncs.com
        """
        fc = 阿里云函数计算(
            endpoint=endpoint,
            accessKeyID=access_key_id,
            accessKeySecret=access_key_secret,
            Timeout=120
        )

        # req = fc.调用http函数("GET", "test1", "fchttp", "", headers={}, params={"a": "aaa"},
        #                   body=bytes('hello_world'.encode('utf-8')))
        # print(req.status_code)
        # print(req.content)


    def test_3(self):
        # 调用事件函数
        环境变量_从文本中加载至系统(读入文本(取运行目录() + "/.env"))
        access_key_id = 取环境变量("access_key_id")
        access_key_secret = 取环境变量("access_key_secret")
        endpoint = 取环境变量("endpoint")
        fc = 阿里云函数计算(
            endpoint=endpoint,
            accessKeyID=access_key_id,
            accessKeySecret=access_key_secret,
            Timeout=120
        )

        data = bytes(json.dumps({
            "key": "hello",
        }).encode("utf-8"))
        for v in range(10):
            print(v)
            ret = fc.调用事件函数("fwmc", "get_list", data, 异步调用=True)
        # ret = fc.调用事件函数("fwmc", "get_list", data, 异步调用=False)
        print(ret.data)
        # print(ret.headers)


    def test_2(self):
        # 列出服务和函数
        环境变量_从文本中加载至系统(读入文本(取运行目录() + "/.env"))

        access_key_id = 取环境变量("access_key_id")
        access_key_secret = 取环境变量("access_key_secret")
        endpoint = 取环境变量("endpoint")

        fc = 阿里云函数计算(
            endpoint=endpoint,
            accessKeyID=access_key_id,
            accessKeySecret=access_key_secret,
            Timeout=120
        )
        print(fc.列出服务().data)
        for v in fc.列出服务().data['services']:
            print("-------------------------------")
            print(repr(v))
            print("服务名称", v['serviceName'])
            print("描述", v['description'])
            print("role", v['role'])
            print("serviceId", v['serviceId'])
            print("createdTime", v['createdTime'])
            print(fc.列出函数(v['serviceName']).data)
            for fcv in fc.列出函数(v['serviceName']).data['functions']:
                print("====================================")
                print(repr(fcv))
                print("函数名称", fcv['functionName'])
                print("描述", fcv['description'])
                print("运行环境", fcv['runtime'])
                print("入口函数", fcv['handler'])
                print("超时时间", fcv['timeout'])
            print("-------------------------------")






"""

.. Hint::
    日期时间操作 符合中国人思维逻辑的日期时间操作实用函数


.. literalinclude:: ../../../pyefun/核心支持库/日期时间操作_test.py
    :language: python
    :caption: 代码示例
    :linenos:

"""

import time
import pendulum
import copy
from .文本操作 import *


def 时间_设置语言包(名称="zh"):
    pendulum.set_locale(名称)


# 文档
# https://pendulum.eustace.io/docs/#instantiation
class 日期时间:
    t = pendulum.DateTime

    def __init__(self, str="now"):
        pass
        if (type(str) == int):
            self.t = pendulum.from_timestamp(str, tz='Asia/Shanghai')
        elif (str != ""):
            self.t = pendulum.parse(str)

    def 到文本(self, format="YYYY-MM-DD HH:mm:ss"):
        return self.t.format(format)

    def 取时间戳(self):
        return self.t.int_timestamp

    def 取日期(self):
        return self.t.to_date_string()

    def 取年(self):
        return self.t.year

    def 取月(self):
        return self.t.month

    def 取日(self):
        return self.t.day

    def 取小时(self):
        return self.t.hour

    def 取分钟(self):
        return self.t.minute

    def 取秒(self):
        return self.t.second

    def 取微妙(self):
        return self.t.microsecond

    def 取每周的第几天(self):
        return self.t.day_of_week

    def 取每年的第几天(self):
        return self.t.day_of_year

    def 取每月的第几周(self):
        return self.t.week_of_month

    def 取每年的第几周(self):
        return self.t.week_of_year

    def 取月天数(self):
        return self.t.days_in_month

    def 设置年月日(self, year=None, month=None, day=None):
        self.t = self.t.on(year=int(year), month=int(month), day=int(day))
        return self

    def 设置时分秒(self, hour=0, minute=0, second=0):
        self.t = self.t.at(hour, minute, second)
        return self

    def 增减日期时间(self, years=0, months=0, weeks=0, days=0, hours=0, minutes=0, seconds=0):
        self.t = self.t.add(years=years,
                            months=months,
                            weeks=weeks,
                            days=days,
                            hours=hours,
                            minutes=minutes,
                            seconds=seconds,
                            )
        return self

    def 取时间间隔(self, datetime):

        d = self.t.diff(datetime, abs=False)
        return {
            "years": d.in_years(),
            "months": d.in_months(),
            "days": d.in_days(),
            "hours": d.in_hours(),
            "minutes": d.in_minutes(),
            "seconds": d.in_seconds(),
            "weeks": d.in_weeks(),
        }

    def 取友好时间(self):
        d = self.t.diff_for_humans()
        return d

    def datetime(self):
        return self.t

    def copy(self):
        return copy.deepcopy(self)

    def __str__(self):
        return self.到文本()


def 取现行时间戳():
    return int(time.time())


def 日期到时间戳(str="now"):
    return 日期时间(str).取时间戳()


def 取现行时间():
    t = 日期时间()
    return t


def 取现行时间2(格式, 自定义=False):
    """
    自定义  %Y-%m-%d %H:%M:%S    (yyyy-MM-dd HH:mm:ss)这种格式不可以
    格式1 = 2020-10-09 22:11:52   格式2 = 2020-10-09 格式3= 22:13:37 格式4= 2020年10月17日18时27分40秒

    %a  # 本地(local) 简化星期名称
    %A  # 本地完整星期名称
    %b  # 本地简化月份名称
    %B  # 本地完整月份名称
    %c  # 本地相应的日期和时间表示
    %d  # 一个月中的第几天（01-31）
    %H  # 一天中的第几个小时（24小时制00-23）
    %I  # 第几个小时（12小时制01-12）
    %j  # 一年中的第几天（001-366）
    %m  # 月份（01-12）
    %M  # 分钟数（00-59）
    %p  # 本地am或pm的相应符
    %S  # 秒（01-60）
    %U  # 一年中的星期数。（00-53 星期天是一个星期的开始）第一个星期天之前的所有天数都放在第0周
    %w  # 一个星期中的第几天（0-6 0是星期天）
    %W  # 和%U基本相同，不同的是%W以星期一为一个星期的开始
    %x  # 本地相应日期
    %X  # 本地相应时间
    %y  # 去掉世纪的年份（00-99）
    %Y  # 完整的年份
    %z  # 时区的名字
    """
    if 自定义:
        return time.strftime(格式, time.localtime())  # 2020-10-09 22:11:52
    if 格式 == 1:
        return time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())  # 2020-10-09 22:11:52
    elif 格式 == 2:
        return time.strftime("%Y-%m-%d", time.localtime())  # 2020-10-09
    elif 格式 == 3:
        return time.strftime("%H:%M:%S", time.localtime())  # 22:13:37
    elif 格式 == 4:
        return time.strftime("%Y{}%m{}%d{}%H{}%M{}%S{}", time.localtime()).format('年', '月', '日', '时', '分',
                                                                                  '秒')  # 2020年10月17日18时27分40秒


def now():
    t = 日期时间()
    return t


def 创建日期时间(timestr):
    t = 日期时间(timestr)
    return t


def 时间迭代(start, end):
    """
    为支持单位range()有：years，months，weeks， days，hours，minutes和seconds
    for i in list.range('months'):
        print(i.to_datetime_string())
    """
    return pendulum.period(pendulum.parse(start), pendulum.parse(end))


def date(format, timestr="now"):
    t = 创建日期时间(timestr)
    format = 子文本替换(format, "Y", str(t.取年()))
    format = 子文本替换(format, "m", str(t.取月()))
    format = 子文本替换(format, "d", str(t.取日()))
    format = 子文本替换(format, "H", str(t.取小时()))
    format = 子文本替换(format, "i", str(t.取分钟()))
    format = 子文本替换(format, "s", str(t.取秒()))
    return format



# pip install pygetwindow

import pygetwindow as gw


class 窗口操作:
    def __init__(self):
        pass

    def 获取窗口(self, 窗口名称):
        窗口 = gw.getWindowsWithTitle(窗口名称)
        if 窗口:
            return 窗口[0]
        else:
            return False
    def 获取所有窗口标题(self):
        return gw.getAllTitles()
    def 获取所有窗口句柄(self):
        return gw.getAllWindows()



if __name__ == "__main__":
    窗口操作 = 窗口操作()
    # 微信窗口 = 窗口操作.获取窗口根据标题("微信")
    # 微信窗口.activate()
    res = 窗口操作.获取所有窗口标题()
    print(res)
    res = 窗口操作.获取所有窗口句柄()
    print(res)







import base64
from io import BytesIO

import cv2
import numpy as np
from PIL import Image, ImageDraw, ImageFont, ImageEnhance, ImageFilter


class 图像操作PIL类(object):
    """
    图像操作类PIL
    本类用于对图像进行操作 使用PIL库
    """

    def __init__(self, obj=None):
        # print(type(obj))
        if type(obj) == str:
            self.img = Image.open(obj)
        else:
            self.img = obj

    def 加载图片(self, image_path):
        self.img = Image.open(image_path)

    def 加载图片从base64(self, base64_str):
        img_io = BytesIO(base64.b64decode(base64_str))
        self.img = Image.open(img_io)
        return self

    def 创建空白图片(self, w, h, color=(255, 255, 255)):
        self.img = Image.new('RGB', (w, h), color)
        return self

    def 绘制图片(self, img, x1=0, y1=0, w=100, h=100):
        self.img.paste(img, (x1, y1, w, h))

    def 到数据(self):
        return np.array(self.img)

    def 显示图片(self):
        data = np.array(self.img)[..., ::-1]
        cv2.imshow("show", data)
        cv2.waitKey(0)
        return self

    # 显示图片 系统打开图片的方式
    def 显示图片2(self):
        self.img.show()

    # 剪裁图片
    def 剪裁(self, x1, y1, w, h):
        self.img = self.img.crop((x1, y1, w, h))
        return self

    # 保存为文件
    def 保存为文件(self, path):
        self.img.save(path)
        return self

    # 二值化
    def 二值化(self, threshold=127):
        self.img = self.img.convert('L')
        self.img = self.img.point(lambda x: 0 if x < threshold else 255, '1')
        return self

    # 绘制矩形
    def 绘制矩形(self, x1, y1, w, h, color=(255, 0, 0), width=1):
        draw = ImageDraw.Draw(self.img)
        draw.rectangle((x1, y1, w, h), outline=color, width=width)
        return self

    # 绘制文本汉字
    def 绘制文本(self, text, x1, y1, color=(255, 0, 0), font_size=20):
        draw = ImageDraw.Draw(self.img)
        font = ImageFont.truetype("msyh.ttc", font_size, encoding="utf-8")
        draw.text((x1, y1), text, color, font)
        return self

    # 绘制填充矩形
    def 绘制填充矩形(self, x1, y1, x2, y2, color=(255, 0, 0), 透明度=0.5):
        h, w = self.img.height, self.img.width
        img_left = self.img.copy()
        draw_left = ImageDraw.Draw(img_left)
        draw_left.polygon([(x1, y1), (x2, y1), (x2, y2), (x1, y2)], outline=color, fill=color)
        img_left = Image.blend(self.img, img_left, 透明度)
        img_show = Image.new('RGB', (w, h), (255, 255, 255))
        img_show.paste(img_left, (0, 0, w, h))
        self.img = np.array(img_show)
        return self

    # 绘制多边形
    def 绘制多边形(self, points, color=(255, 0, 0)):
        draw_left = ImageDraw.Draw(self.img)
        draw_left.polygon(np.array(points, dtype=np.float32), outline=color)
        return self

    # 绘制填充多边形
    def 绘制填充多边形(self, points, color=(255, 0, 0), 透明度=0.5):
        h, w = self.img.height, self.img.width
        img_left = self.img.copy()
        draw_left = ImageDraw.Draw(img_left)
        draw_left.polygon(np.array(points, dtype=np.float32), outline=color, fill=color)
        img_left = Image.blend(self.img, img_left, 透明度)
        img_show = Image.new('RGB', (w, h), (255, 255, 255))
        img_show.paste(img_left, (0, 0, w, h))
        self.img = np.array(img_show)
        return self

    # 到base64
    def 到base64(self):
        img_io = BytesIO()
        self.img.save(img_io, 'JPEG')
        img_io.seek(0)
        return base64.b64encode(img_io.getvalue()).decode()

    # 黑白颜色反转
    def 颜色翻转(self):
        self.img = self.img.convert('L')
        return self

    def 图像翻转左右(self):
        self.img = self.img.transpose(Image.FLIP_LEFT_RIGHT)
        return self

    def 图像翻转上下(self):
        self.img = self.img.transpose(Image.FLIP_TOP_BOTTOM)
        return self

    def 图像翻转180(self):
        self.img = self.img.transpose(Image.ROTATE_180)
        return self

    def 图像翻转90(self):
        self.img = self.img.transpose(Image.ROTATE_90)
        return self

    def 图像翻转270(self):
        self.img = self.img.transpose(Image.ROTATE_270)
        return self

    @property
    def 宽度(self):
        return self.img.width

    @property
    def 高度(self):
        return self.img.height

    def 取所有颜色(self):
        return self.img.getcolors()

    def 绘制点(self, x1, y1, color=(255, 0, 0)):
        draw = ImageDraw.Draw(self.img)
        draw.point((x1, y1), color)
        return self

    def 取颜色(self, x1, y1):
        return self.img.getpixel((x1, y1))

    # 绘制圆形
    def 绘制圆形(self, x1, y1, r, color=(255, 0, 0), width=1):
        draw = ImageDraw.Draw(self.img)
        draw.ellipse((x1 - r, y1 - r, x1 + r, y1 + r), outline=color, width=width)
        return self

    # 绘制填充圆形
    def 绘制填充圆形(self, x1, y1, r, color=(255, 0, 0), 透明度=0.5):
        h, w = self.img.height, self.img.width
        img_left = self.img.copy()
        draw_left = ImageDraw.Draw(img_left)
        draw_left.ellipse((x1 - r, y1 - r, x1 + r, y1 + r), outline=color, fill=color)
        img_left = Image.blend(self.img, img_left, 透明度)
        img_show = Image.new('RGB', (w, h), (255, 255, 255))
        img_show.paste(img_left, (0, 0, w, h))
        self.img = np.array(img_show)
        return self

    # 增强对比度
    def 增强对比度(self, 增强倍数=1.5):
        self.img = ImageEnhance.Contrast(self.img).enhance(增强倍数)
        return self

    # 增强亮度
    def 增强亮度(self, 增强倍数=1.5):
        self.img = ImageEnhance.Brightness(self.img).enhance(增强倍数)
        return self

    # 重置图片大小
    def 重置图片大小(self, 宽度, 高度):
        self.img = self.img.resize((宽度, 高度))
        return self
    # 缩放
    def 缩放(self, 缩放倍数):
        self.img = self.img.resize((int(self.img.width * 缩放倍数), int(self.img.height * 缩放倍数)))
        return self

    # 清晰度
    def 清晰度(self, 增减倍数=1.5):
        self.img = ImageEnhance.Sharpness(self.img).enhance(增减倍数)
        return self

    # 高斯模糊
    def 高斯模糊(self, 模糊半径=10):
        self.img = self.img.filter(ImageFilter.GaussianBlur(模糊半径))
        return self

    # 边缘检测
    def 边缘检测(self):
        self.img = self.img.filter(ImageFilter.EDGE_ENHANCE_MORE)
        return self

    # 轮廓
    def 轮廓(self):
        self.img = self.img.filter(ImageFilter.CONTOUR)
        return self




import unittest

from pyefun import *


class TesteRequestsUtil(unittest.TestCase):
    def test2(self):
        pass
        print("网址_取域名", 网址_取域名("http://www.example.com"))

    def test_1(self):
        pass
        import ssl
        ssl._create_default_https_context = ssl._create_unverified_context
        import warnings
        warnings.simplefilter('ignore', ResourceWarning)

        ehttp = eRequests(cookies文件路径="C:/123/1.cookie", 全局头信息="""
        User-Agent: quanjutouxinxi
        Cookie: a1=1111111111111; b1=22222222222; 
        """)
        # ehttp.关闭调试信息()
        ehttp.设置全局HTTP代理("127.0.0.1:11111")

        返回文本 = ehttp.访问("http://127.0.0.1:9000/1.php", 附加头信息="""Cookie: aa=bbbbbbbbbbbb; cc=dddd; 
        xxxxxxxxxxxx: zuigaoyouxianji
""", 允许重定向=True)
        print(返回文本.内容)
        # print(返回文本.文本)
        # print(返回文本.头信息)
        # print(返回文本.cookies)
        # print(返回文本.Response.headers['Location'])
        返回文本 = ehttp.访问("http://127.0.0.1:9000/1.php", 不使用代理访问=True)
        返回文本 = ehttp.访问("http://127.0.0.1:9000/1.php", 允许重定向=False)

    def test_POST(self):
        ehttp = eRequests(cookies文件路径="C:/123/1.cookie")
        ehttp.设置全局HTTP代理("127.0.0.1:11111")
        返回文本 = ehttp.post("http://127.0.0.1:9000/post.php", 发送文本="a=1&b=2&c=3")
        print(返回文本.文本)
        返回文本 = ehttp.post("http://127.0.0.1:9000/post.php", 发送文本={"aa": 1, "bb": 2, "cc": 3})
        print(返回文本.文本)
        # 字节集 = 读入文件("C:/123/share.png")
        # 返回文本 = ehttp.post("http://127.0.0.1:9000/post.php", 发送文本={"aa": 1, "bb": 2, "cc": 3},
        #                   上传文件={'upload': ('code.png', 字节集, 'image/png')})
        # print(返回文本.文本)

    def setUp(self) -> None:
        # 屏蔽一堆警告
        屏蔽Requests中的警告信息()

    def test_header(self):
        headers = {
            'Connection': 'keep-alive',
            'Cache-Control': 'max-age=0',
            'sec-ch-ua': '" Not A;Brand";v="99", "Chromium";v="99", "Google Chrome";v="99"',
            'sec-ch-ua-mobile': '?0',
            'sec-ch-ua-platform': '"macOS"',
            'Upgrade-Insecure-Requests': '1',
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/99.0.4844.84 Safari/537.36',
            'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9',
            'Sec-Fetch-Site': 'same-origin',
            'Sec-Fetch-Mode': 'navigate',
            'Sec-Fetch-User': '?1',
            'Sec-Fetch-Dest': 'document',
            'Referer': 'https://fanyi.baidu.com/',
            'Accept-Language': 'zh-CN,zh;q=0.9,en;q=0.8',
        }
        ehttp = eRequests(全局头信息=headers)
        ehttp = eRequests()
        url = 'https://fanyi.baidu.com'
        ehttp.get(url, 附加头信息=headers)



import ppppocr

ocr = ppppocr.ppppOcr()


def 通用文字识别(图片路径):
    dt_boxes, rec_res = ocr.ocr(图片路径)
    return dt_boxes, rec_res


def 通用文字识别获取文字(图片路径):
    dt_boxes, rec_res = ocr.ocr(图片路径)
    return ocr.toText(rec_res)


def 通用文字识别获取Json(图片路径):
    dt_boxes, rec_res = ocr.ocr(图片路径)
    return ocr.toJson(dt_boxes, rec_res)



# ========== 核心支持库命令 ==========
from pyefun.核心支持库.算数运算 import *
from pyefun.核心支持库.数组操作 import *
from pyefun.核心支持库.编码转换 import *
from pyefun.核心支持库.磁盘操作 import *
from pyefun.核心支持库.公用函数 import *
from pyefun.核心支持库.文本操作 import *
from pyefun.核心支持库.系统处理 import *
from pyefun.核心支持库.日期时间操作 import *
from pyefun.核心支持库.类型转换 import *
from pyefun.核心支持库.时钟 import *
# ========== 核心支持库命令 ==========

# ========== 核心易函数支持库 ==========
from pyefun.核心易函数支持库.实用函数 import *
from pyefun.核心易函数支持库.文本操作实用函数 import *
from pyefun.核心易函数支持库.正则表达式 import *
from pyefun.核心易函数支持库.正则表达式实用函数 import *
from pyefun.核心易函数支持库.网络请求 import *  # 易函数易用的请求模块
from pyefun.核心易函数支持库.时间统计 import *
from pyefun.核心易函数支持库.线程操作 import *
from pyefun.核心易函数支持库.进程池 import *
from pyefun.核心易函数支持库.json函数 import *
from pyefun.核心易函数支持库.环境变量 import *
from pyefun.核心易函数支持库.配置项 import *
from pyefun.核心易函数支持库.网络实用函数 import *
# ========== 核心易函数支持库 ==========


# ========== 模块 ==========
# from pyefun.模块.javascript引擎 import *  # 执行js脚本
# from pyefun.模块.二维码 import *  # 二维码解析
# from pyefun.模块.剪切板操作 import *  # 剪切板操作
# from pyefun.模块.协程池 import *  # 原生协程池
# from pyefun.模块.协程池Gevent import *  # Gevent协程池
# from pyefun.模块.窗口操作 import *  # 窗口操作
# from pyefun.模块.进度显示 import *  # 进度显示
# from pyefun.模块.缓存工具 import *  # 缓存工具
# from pyefun.模块.邮件 import *  # 邮件
# from pyefun.模块.定时任务 import *  # 定时任务
# from pyefun.模块.winapi import *  # window api
# from pyefun.模块.通用实用函数 import *  # ubelt 封装的实用函数
# ========== 模块 ==========

# ========== 编码解码 ==========
# from pyefun.编码解码.gzip解压缩 import * # gzip 解压缩
# from pyefun.编码解码.zlib解压缩 import * # zlib 解压缩
# from pyefun.编码解码.zip解压缩 import * # zip 解压缩
# from pyefun.编码解码.binary编码解码 import * # 二进制 编码解码
# from pyefun.编码解码.base64编码解码 import * # base64 编码解码
# from pyefun.编码解码.url编码解码 import * # URL 编码解码
# ========== 编码解码 ==========

# ========== 数据库 ==========
# from pyefun.数据库.redis工具类 import * # redis
# from pyefun.数据库.mysql数据库 import * # mysql数据库
# from pyefun.数据库.mongo数据库 import * # mongo数据库
# from pyefun.数据库.peeweeUtil import * # peeweeUtil
# ========== 数据库 ==========

# ========== excel操作 ==========
# from pyefun.excel.excel_xlwr import * # excel操作模块 选其中一个使用即可
# from pyefun.excel.excel_openpyxl import * # excel操作模块 选其中一个使用即可
# ========== excel操作 ==========

# ========== 浏览器自动化操作 ==========
# from pyefun.seleniumUtil import * # 浏览器自动化测试
# ========== 浏览器自动化操作 ==========

# ========== 阿里云SDK ==========
# from pyefun.阿里云SDK.fc.fc import *  # 阿里云函数计算操作
# from pyefun.阿里云SDK.oss.oss import *  # 阿里云oss操作sdk
# ========== 阿里云SDK ==========

# ========== 人工智能 ==========
# from pyefun.人工智能.ONNX分类模型推理 import *  # ONNX分类模型推理
# from pyefun.人工智能.通用文字识别 import *  # 提供通用的中文文字识别功能
# ========== 人工智能 ==========

# ========== 图像处理 ==========
# from pyefun.图像处理 import *  # 图片处理 cv2 以及 pil 的工具类
# from pyefun.图像处理.图像处理 import * # opencv 工具类
# from pyefun.图像处理.图像处理PIL import * # pil 工具类
# ========== 图像处理 ==========


__version__ = '1.2.2'



# -*- coding:utf-8 -*-
from win32api import GetSystemMetrics
import win32gui, win32con, win32api, os, re, subprocess
import win32clipboard as w
from pyefun import *

# 字母键
按键_A, 按键_B, 按键_C, 按键_D, 按键_E, 按键_F, 按键_G, 按键_H, 按键_I, 按键_J, 按键_K, 按键_L, 按键_M, 按键_N, 按键_O, 按键_P, 按键_Q, 按键_R, 按键_S, 按键_T, 按键_U, 按键_V, 按键_W, 按键_X, 按键_Y, 按键_Z = 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 82, 83, 84, 85, 86, 87, 88, 89, 90
# 字母上的数字键(左边)
按键_0_, 按键_1_, 按键_2_, 按键_3_, 按键_4_, 按键_5_, 按键_6_, 按键_7_, 按键_8_, 按键_9_ = 48, 49, 50, 51, 52, 53, 54, 55, 56, 57
# 数字键(右边)
按键_0, 按键_1, 按键_2, 按键_3, 按键_4, 按键_5, 按键_6, 按键_7, 按键_8, 按键_9 = 96, 97, 98, 99, 100, 101, 102, 103, 104, 105
# 功能键
按键_F1, 按键_F2, 按键_F3, 按键_F4, 按键_F5, 按键_F6, 按键_F7, 按键_F8, 按键_F9, 按键_F10, 按键_F11, 按键_F12 = 112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122, 123
# 其它键
按键_乘, 按键_加, 按键_小回车, 按键_减, 按键_点, 按键_除 = 106, 107, 108, 109, 110, 111
按键_退格, 按键_TAB, 按键_清除键, 按键_回车, 按键_SHIFT, 按键_CONTROL, 按键_ALT, 按键_大小写键, 按键_ESC, 按键_空格, 按键_PAGE_UP, 按键_PAGE_DOWN, 按键_END, 按键_HOME, 按键_向左, 按键_向右 = 8, 9, 12, 13, 16, 17, 18, 20, 27, 32, 33, 34, 35, 36, 37, 38
按键_向右, 按键_向下, 按键_INS, 按键_DEL, 按键_NUM, 按键_冒号键, 按键_等号键, 按键_逗号键, 按键_下划线键, 按键_句号键, 按键_问号键, 按键_波浪键, 按键_左括号, 按键_斜杠键, 按键_右括号, 按键_引号键 = 39, 40, 45, 46, 144, 186, 187, 188, 189, 190, 191, 192, 219, 220, 221, 222
按键_音量加, 按键_音量减, 按键_停止, 按键_静音, 按键_浏览器, 按键_邮件, 按键_搜索, 按键_搜餐 = 175, 174, 179, 173, 172, 180, 170, 171
按键_WIN = 91


@异常处理返回类型逻辑型
def 窗口_置父窗口(子窗口句柄, 父窗口句柄):
    '将子窗口嵌入父窗口内'
    return win32gui.SetParent(子窗口句柄, 父窗口句柄)


@异常处理返回类型逻辑型
def 鼠标_移动(x, y):
    '在桌面上移动鼠标位置'
    win32api.SetCursorPos((x, y))


@异常处理返回类型逻辑型
def 鼠标_点击(按键=0, 类型=0, x=0, y=0):
    '''
    模拟鼠标点击
    :param 按键: 0.左键 1.中键 2.右键
    :param 类型: 0.单击 1.按下 2.弹起
    :return:
    '''
    代码 = {(0, 1): 2, (0, 2): 4, (1, 1): 32, (1, 2): 64, (2, 1): 8, (2, 2): 16}
    按键类型 = {1: (32, 64), 2: (8, 16)}
    if not (按键, 类型) in 代码:
        键码 = 按键类型[按键] if 按键 in 按键类型 else (2, 4)
        win32api.mouse_event(键码[0], 0, 0, 0, 0)
        win32api.mouse_event(键码[1], 0, 0, 0, 0)
    else:
        win32api.mouse_event(代码[(按键, 类型)], 0, 0, 0, 0)


@异常处理返回类型逻辑型
def 键盘_点击(键码, 类型=0):
    '''
    :param 键码: 可直接使用 按键_按键名
    :param 类型: 0.单击 1.按下 2.弹起
    :return:
    '''
    if 类型 == 1:
        win32api.keybd_event(键码, 0, 0, 0)
    elif 类型 == 2:
        win32api.keybd_event(键码, 0, win32con.KEYEVENTF_KEYUP, 0)
    else:
        win32api.keybd_event(键码, 0, 0, 0)
        win32api.keybd_event(键码, 0, win32con.KEYEVENTF_KEYUP, 0)


@异常处理返回类型逻辑型
def 窗口_取窗口句柄(类名=None, 标题=None):
    '返回第一个找到的句柄'
    return win32gui.FindWindow(类名, 标题)


@异常处理返回类型逻辑型
def 窗口_取坐标处窗口句柄(x, y):
    return win32gui.WindowFromPoint((x, y))


@异常处理返回类型逻辑型
def 窗口_取窗口类名(窗口句柄):
    return win32gui.GetClassName(窗口句柄)


@异常处理返回类型逻辑型
def 窗口_取窗口标题(窗口句柄):
    return win32gui.GetWindowText(窗口句柄)


@异常处理返回类型逻辑型
def 窗口_取窗口矩形(窗口句柄):
    '成功返回窗口 左边 顶边 右边 底边'
    return win32gui.GetWindowRect(窗口句柄)


@异常处理返回类型逻辑型
def 窗口_枚举子窗口句柄(窗口句柄):
    '成功返回子窗口句柄列表'
    hwndChildList = []
    win32gui.EnumChildWindows(窗口句柄, lambda hwnd, param: param.append(hwnd), hwndChildList)
    return hwndChildList


@异常处理返回类型逻辑型
def 窗口_发送信息(窗口句柄, 消息类型, 参数1=0, 参数2=0):
    return win32gui.PostMessage(窗口句柄, 消息类型, 参数1, 参数2)


@异常处理返回类型逻辑型
def 窗口_发送信息2(窗口句柄, 消息类型, 参数1=0, 参数2=0):
    return win32gui.SendMessage(窗口句柄, 消息类型, 参数1, 参数2)


@异常处理返回类型逻辑型
def 窗口_发送文本(窗口句柄, 内容):
    return win32api.SendMessage(窗口句柄, win32con.WM_SETTEXT, 0, 内容)


@异常处理返回类型逻辑型
def 窗口_取窗口文本长度(窗口句柄):
    return win32api.SendMessage(窗口句柄, win32con.WM_GETTEXTLENGTH, 0, 0) + 1


@异常处理返回类型逻辑型
def 窗口_取窗口文本内容(窗口句柄):
    bufSize = win32api.SendMessage(窗口句柄, win32con.WM_GETTEXTLENGTH, 0, 0) + 1
    strBuf = win32gui.PyMakeBuffer(bufSize)
    length = win32gui.SendMessage(窗口句柄, win32con.WM_GETTEXT, bufSize, strBuf)
    address, length = win32gui.PyGetBufferAddressAndLen(strBuf)
    text = win32gui.PyGetString(address, length)
    return text[:-1]


@异常处理返回类型逻辑型
def 窗口_关闭窗口(窗口句柄):
    return win32gui.PostMessage(窗口句柄, win32con.WM_CLOSE, 0, 0)


@异常处理返回类型逻辑型
def 窗口_是否最小化(窗口句柄):
    return bool(win32gui.IsIconic(窗口句柄))


@异常处理返回类型逻辑型
def 窗口_最大化(窗口句柄):
    return bool(win32gui.ShowWindow(窗口句柄, win32con.SW_MAXIMIZE))


@异常处理返回类型逻辑型
def 窗口_最大化2(窗口句柄):
    '激活窗口并将其最大化'
    return bool(win32gui.ShowWindow(窗口句柄, win32con.SW_SHOWMAXIMIZED))


@异常处理返回类型逻辑型
def 窗口_最小化(窗口句柄):
    return bool(win32gui.ShowWindow(窗口句柄, win32con.SW_MINIMIZE))


@异常处理返回类型逻辑型
def 窗口_最小化2(窗口句柄):
    '激活窗口并将其最小化'
    return bool(win32gui.ShowWindow(窗口句柄, win32con.SW_SHOWMINIMIZED))


@异常处理返回类型逻辑型
def 窗口_最小化3(窗口句柄):
    '窗口最小化，激活窗口仍然维持激活状态'
    return bool(win32gui.ShowWindow(窗口句柄, win32con.SW_SHOWMINNOACTIVE))


@异常处理返回类型逻辑型
def 窗口_隐藏窗口(窗口句柄):
    return bool(win32gui.ShowWindow(窗口句柄, win32con.SW_HIDE))


@异常处理返回类型逻辑型
def 窗口_显示窗口(窗口句柄):
    return bool(win32gui.ShowWindow(窗口句柄, win32con.SW_SHOW))


@异常处理返回类型逻辑型
def 窗口_显示窗口2(窗口句柄):
    '以原尺寸恢复显示'
    return bool(win32gui.ShowWindow(窗口句柄, win32con.SW_RESTORE))


@异常处理返回类型逻辑型
def 窗口_显示窗口3(窗口句柄):
    '以窗口原来的状态显示窗口。激活窗口仍然维持激活状态'
    return bool(win32gui.ShowWindow(窗口句柄, win32con.SW_SHOWNA))


@异常处理返回类型逻辑型
def 窗口_显示窗口4(窗口句柄):
    '以窗口最近一次的大小和状态显示窗口。激活窗口仍然维持激活状态'
    return bool(win32gui.ShowWindow(窗口句柄, win32con.SW_SHOWNOACTIVATE))


@异常处理返回类型逻辑型
def 窗口_显示窗口5(窗口句柄):
    '激活并显示一个窗口。如果窗口被最小化或最大化，系统将其恢复到原来的尺寸和大小。应用程序在第一次显示窗口的时候应该指定此标志'
    return bool(win32gui.ShowWindow(窗口句柄, win32con.SW_SHOWNORMAL))


@异常处理返回类型逻辑型
def 窗口_调整(窗口句柄, 左边, 顶边, 宽度, 高度):
    '移动窗口 调整宽度高度'
    return win32gui.MoveWindow(窗口句柄, 左边, 顶边, 宽度, 高度, True)


@异常处理返回类型逻辑型
def 窗口_调整2(窗口句柄, 左边, 顶边, 宽度, 高度):
    '移动窗口 调整宽度高度'
    return win32gui.SetWindowPos(窗口句柄, win32con.HWND_TOPMOST, 左边, 顶边, 宽度, 高度, win32con.SWP_SHOWWINDOW)


@异常处理返回类型逻辑型
def 窗口_置焦点(窗口句柄):
    '指定句柄设置为前台，也就是激活'
    return win32gui.SetForegroundWindow(窗口句柄)


@异常处理返回类型逻辑型
def 窗口_置后台(窗口句柄):
    '设置为后台'
    return win32gui.SetBkMode(窗口句柄, win32con.TRANSPARENT)


def _MyCallback(hwnd, extra):  # 回调
    windows = extra
    temp = []
    temp.append(hex(hwnd))
    temp.append(win32gui.GetClassName(hwnd))
    temp.append(win32gui.GetWindowText(hwnd))
    windows[hwnd] = temp


@异常处理返回类型逻辑型
def 窗口_取所有顶级窗口句柄():
    '返回一个字典包含 key为句柄 值为列表[16进制的啥,类名,标题]'
    windows = {}
    win32gui.EnumWindows(_MyCallback, windows)
    return windows


@异常处理返回类型逻辑型
def 取剪辑版文本():
    '获取完需要自己解码,常规用utf8带汉字的用gbk等'
    w.OpenClipboard()
    d = w.GetClipboardData(win32con.CF_TEXT)
    w.CloseClipboard()
    return d


@异常处理返回类型逻辑型
def 置剪辑版文本(内容):
    w.OpenClipboard()
    w.EmptyClipboard()
    w.SetClipboardText(内容)
    w.CloseClipboard()
    return True


@异常处理返回类型逻辑型
def 运行程序(文件路径, 参数='', 初始目录='', 显示窗口=True):
    '启动/运行一个文件/程序'
    return win32api.ShellExecute(1, 'open', 文件路径, 参数, 初始目录, 显示窗口)


@异常处理返回类型逻辑型
def 信息框(内容, 标题='提示', 类型=0, 图标=0):
    '''
    api创建的信息框
    :param 标题:
    :param 内容:
    :param 类型: 0.通常   1.确定 取消   2.是 否     3.是 否 取消   4.终止 重试 忽略
    :param 图标: 0.没有   1.蓝色提示   2.黄色感叹   3.红色叉叉
    :return: 出错返回0，正常返回值：1.确定 2.取消 3.中止 4.重试 5.忽略 6.是  7.否
    '''
    按钮类型 = {0: win32con.MB_OK, 1: win32con.MB_OKCANCEL, 2: win32con.MB_YESNO, 3: win32con.MB_YESNOCANCEL,
            4: win32con.MB_ABORTRETRYIGNORE}
    图标类型 = {0: 0, 1: win32con.MB_ICONASTERISK, 2: win32con.MB_ICONEXCLAMATION, 3: win32con.MB_ICONERROR}
    return win32api.MessageBox(None, 内容, 标题, 按钮类型[类型] | 图标类型[图标])


@异常处理返回类型逻辑型
def 系统_建立关联文件(后缀, 程序路径, 图标=None):
    '后缀：.pyec,程序路径:完整的路径'
    程序名 = os.path.splitext(程序路径)[0]
    图标 = 图标 if 图标 else 程序路径
    # 创建
    key = win32api.RegOpenKey(win32con.HKEY_CLASSES_ROOT, '', 0, win32con.KEY_READ)
    win32api.RegCreateKey(key, 后缀)
    win32api.RegCreateKey(key, 程序名 + '\\BrowserFlags')
    win32api.RegCreateKey(key, 程序名 + '\\EditFlags')
    win32api.RegCreateKey(key, 程序名 + '\\DefaultIcon\\')
    win32api.RegCreateKey(key, 程序名 + '\\shell\\')
    win32api.RegCreateKey(key, 程序名 + '\\shell\\open\\command\\')
    win32api.RegCloseKey(key)
    # 写入默认值
    key = win32api.RegOpenKey(win32con.HKEY_CLASSES_ROOT, '', 0, win32con.KEY_READ)
    win32api.RegSetValue(key, '.pyec', win32con.REG_SZ, 程序名)
    win32api.RegSetValue(key, 程序名 + '\\BrowserFlags', win32con.REG_SZ, "8")
    win32api.RegSetValue(key, 程序名 + '\\EditFlags', win32con.REG_SZ, "0")
    win32api.RegSetValue(key, 程序名 + '\\DefaultIcon\\', win32con.REG_SZ, '"{}"'.format(图标))
    win32api.RegSetValue(key, 程序名 + '\\shell\\', win32con.REG_SZ, 'open')
    win32api.RegSetValue(key, 程序名 + '\\shell\\open\\command\\', win32con.REG_SZ, r'{} "%1"'.format(程序路径))
    win32api.RegCloseKey(key)
    return True


@异常处理返回类型逻辑型
def 系统_取Python目录():
    try:
        key = win32api.RegOpenKey(win32con.HKEY_LOCAL_MACHINE,
                                  'SYSTEM\\CurrentControlSet\\Control\\Session Manager\\Environment', 0,
                                  win32con.KEY_ALL_ACCESS)
        版本 = win32api.RegQueryValueEx(key, 'Path')  # 遍历注册表下目录
        win32api.RegCloseKey(key)
        匹配 = re.findall(r'\b[A-Z]:\\.*?python\b', 版本[0])
        if 匹配:
            return 匹配[0] + "\\"
    except:
        pass

    try:
        p = subprocess.Popen("where python", stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        结果 = p.stdout.readlines()[0].decode('utf8')
        if 结果.find("python") != -1:
            return os.path.split(结果.split("\n")[0])[0] + "\\"
    except:
        pass

    try:
        key = win32api.RegOpenKey(win32con.HKEY_CURRENT_USER, 'Software\\Python\\PythonCore', 0,
                                  win32con.KEY_ALL_ACCESS)
        版本 = win32api.RegEnumKeyEx(key)[0][0]  # 遍历注册表下目录
        win32api.RegCloseKey(key)
        key = win32api.RegOpenKey(win32con.HKEY_CURRENT_USER,
                                  'Software\\Python\\PythonCore\\{}\\InstallPath'.format(版本), 0,
                                  win32con.KEY_ALL_ACCESS)
        目录 = win32api.RegQueryValueEx(key, '')[0]
        win32api.RegCloseKey(key)
        if 目录.endswith('\\'):
            return 目录
        return 目录 + "\\"
    except:
        return ''


@异常处理返回类型逻辑型
def 文件_置隐藏Ex(路径):
    '隐藏文件'
    win32api.SetFileAttributes(路径, win32con.FILE_ATTRIBUTE_HIDDEN)
    return True


@异常处理返回类型逻辑型
def 文件_置只读Ex(路径):
    '修改文件为只读'
    win32api.SetFileAttributes(路径, win32con.FILE_ATTRIBUTE_READONLY)
    return True


@异常处理返回类型逻辑型
def 文件_置系统文件Ex(路径):
    '修改文件为系统文件'
    win32api.SetFileAttributes(路径, win32con.FILE_ATTRIBUTE_SYSTEM)
    return True


@异常处理返回类型逻辑型
def 文件_置正常Ex(路径):
    '文件被隐藏等可以使用这个还原显示'
    win32api.SetFileAttributes(路径, win32con.FILE_ATTRIBUTE_NORMAL)
    return True


@异常处理返回类型逻辑型
def 宽带_拨号(宽带名称, 宽带账号, 宽带密码):
    dos = 'rasdial {} {} {}'.format(宽带名称, 宽带账号, 宽带密码)
    return False if os.system(dos) else True


@异常处理返回类型逻辑型
def 宽带_断开(宽带名称):
    dos = 'rasdial {} /disconnect'.format(宽带名称)
    return False if os.system(dos) else True


def 取屏幕宽度():
    return GetSystemMetrics(0)


def 取屏幕高度():
    return GetSystemMetrics(1)


# 取鼠标水平位置
# 取鼠标垂直位置
def 取鼠标位置():
    return pyautogui.position()



import unittest

from .json函数 import *


class Testjson函数(unittest.TestCase):

    def test_1(self):
        pass
        res = json到文本({"a": 1, "b": 2})
        print(res)
        self.assertEqual(res, '{"a":1,"b":2}')
        res = json加载('{"a":1,"b":2}')
        print(res)
        self.assertEqual(res, {"a": 1, "b": 2})



import unittest

from .正则表达式实用函数 import *


class TestRegexpUtil(unittest.TestCase):

    def test1(self):
        pass
        结果 = 正则匹配手机号码("13800138000")
        self.assertEqual(结果, "13800138000")
        结果 = 正则匹配邮箱("123@qq.com")
        self.assertEqual(结果, "123@qq.com")
        结果 = 正则匹配身份证号码("441234198812093019")
        self.assertEqual(结果, "441234198812093019")
        结果 = 正则匹配银行卡号("6222021001000123456789")
        self.assertEqual(结果, "6222021001000123456789")
        结果 = 正则匹配邮政编码("100100")
        self.assertEqual(结果, "100100")
        结果 = 正则匹配IP地址("192.168.1.1")
        self.assertEqual(结果, "192.168.1.1")
        结果 = 正则匹配URL("http://www.baidu.com")
        self.assertEqual(结果, "http://www.baidu.com")
        结果 = 正则匹配用户名("admin")
        self.assertEqual(结果, "admin")
        结果 = 正则匹配密码("adminAdmin123")
        self.assertEqual(结果, "adminAdmin123")
        结果 = 正则匹配中文("中文")
        self.assertEqual(结果, "中文")
        结果 = 正则匹配英文("abc")
        self.assertEqual(结果, "abc")
        结果 = 正则匹配数字("123")
        self.assertEqual(结果, "123")



import wx
from .wxControl import *
from .wxItemContainer import *


class 列表框(wx.ListBox, 列表项目操作类, 公用方法):
    pass

    @组件_异常检测
    def 取消指定选中项(self, 索引):
        '在列表框中取消选择一个项目。'
        return self.Deselect(索引)

    @组件_异常检测
    def 保证显示(self, 索引):
        '确保当前显示具有给定索引的项目。'
        return self.EnsureVisible(索引)

    def 取可见项目数(self):
        '返回可以垂直放入列表框可见区域的项目数。'
        return self.GetCountPerPage()

    def 取选中范围索引(self):
        '返回一个列表包含所有选中项索引,用当前所选项目的位置填充一个整数数组。'
        return self.GetSelections()

    def 取首个可见项索引(self):
        '返回最顶部可见项目的索引。'
        return self.GetTopItem()

    @组件_异常检测
    def 取指定坐标索引(self, 左边, 顶边):
        '返回列表框内指定坐标处项目索引'
        return self.HitTest(左边, 顶边)

    @组件_异常检测
    def 项目是否选中(self, 索引):
        return self.InsertItems(索引)

    def 表项是否按字母排序(self):
        return self.IsSorted()

    @组件_异常检测
    def 置顶指定项(self, 索引):
        '将指定的项目设置为第一个可见项目。'
        self.SetFirstItem(索引)

    @组件_异常检测
    def 置项目背景色(self, 索引, 颜色):
        '在列表框中设置项目的背景色。仅在MSW上且wx.LB_OWNERDRAW设置了标志时有效。'
        self.SetItemBackgroundColour(索引, 颜色)
        self.Refresh()

    @组件_异常检测
    def 置项目前景色(self, 索引, 颜色):
        '在列表框中设置项目的前景色。仅在MSW上且wx.LB_OWNERDRAW设置了标志时有效。'
        self.SetItemForegroundColour(索引, 颜色)
        self.Refresh()

    @组件_异常检测
    def 置项目字体(self, 索引, 字体):
        '在列表框中设置项目的字体。仅在MSW上且wx.LB_OWNERDRAW设置了标志时有效。'
        self.SetItemFont(索引, 字体)
        self.Refresh()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time : 2020-11-21 21:12
# @Author : Nuonuo
# @Site : 
# @File : 表格.py
# @Software: PyCharm


import openpyxl  # 表格


# 表格openpyxl================================================================
class 表格:
    def __init__(self):
        print('初始化 openpyxl 成功')
        print('行数0开始   列数1开始')

    # '''
    # 在使用openpyxl模块前，需要了解openpyx中名称的概念：
    # 在openpyxl中，主要用到三个概念：Workbooks，Sheets，Cells
    # Workbook：指一个excel工作表；
    # Sheet：指工作表中的一张表；
    # Cell：指表中的一个单元格。
    # openpyx在读或写时顺序为：打开Workbook，定位Sheet，操作Cell
    # '''
    def 创建_工作簿对象(self):
        '''如果写入中文为乱码，可添加参数encoding = '''
        self.__wb = openpyxl.Workbook()
        print('初始化workbok成功')
        print(self.__wb)

    def 创建_工作表(self, 工作表名称, 位置=None):
        '默认在最后添加一个工作表 也可以选择位置插入  openpyxl模块暂不支持xls格式文件'
        self.__sheet = self.__wb.create_sheet(工作表名称, 位置)

    def 读取_工作簿对象(self, path, 只读=False):
        if path[-4:] == 'xlsx':
            self.__wb = openpyxl.load_workbook(path, read_only=只读)
            print('初始化workbok成功')
            print(self.__wb)
            self.设置_默认工作表对象()
        else:
            print(path, '只支持xlsx格式文件')

    def 设置_默认工作表对象(self):
        '默认获取第一个表格对象'
        self.__sheet = self.__wb[self.__wb.sheetnames[0]]
        print('初始化worksheet成功')
        print(self.__sheet)

    def 查看_所有工作表名字(self):
        return self.__wb.sheetnames

    def 选择表(self, 表名):
        ''' #sheet1 = book.worksheets[n]、sheet2 = book['工作表名称']、sheet3 = book[book.sheetnames[n]]'''
        print('初始化worksheet成功')
        self.__sheet = self.__wb[表名]

    def 获取_当前工作表_名称(self):
        return self.__sheet.title

    def 获取_关键字_行内容(self, 关键字, 关键列, 只取id=False):
        '遍历查询调用内部行数'
        最大行数 = self.__sheet.max_row
        data = None
        for i in range(1, 最大行数 + 1):
            cell_value = self.__sheet.cell(row=i, column=关键列).value
            if cell_value == 关键字:
                if 只取id:
                    data = i
                else:
                    data = self.获取_某行所有值(i)
                break
            # data.append(cell_value)
        if data == None:
            return '没有查询数据'
        return data

    def 获取_关键字_列内容(self, 关键字, 关键列, 只取id=False):
        '遍历查询调用内部行数'
        最大列数 = self.__sheet.max_column
        data = None
        for i in range(1, 最大列数 + 1):
            cell_value = self.__sheet.cell(row=关键列, column=i).value
            if cell_value == 关键字:
                if 只取id:
                    data = i
                else:
                    data = self.获取_某列的所有值(i)
                break
            # data.append(cell_value)
        if data == None:
            return '没有查询数据'
        return data

    def 获取_某列的所有值(self, 列):
        rows = self.__sheet.max_row
        column_data = []
        for i in range(1, rows + 1):
            cell_value = self.__sheet.cell(row=i, column=列).value
            column_data.append(cell_value)
        return column_data

    def 获取_某行所有值(self, 行):
        columns = self.__sheet.max_column
        row_data = []
        for i in range(1, columns + 1):
            cell_value = self.__sheet.cell(row=行, column=i).value
            row_data.append(cell_value)
        return row_data

    def 重命名_当前工作表(self, 新表名):
        self.__sheet.title = 新表名

    def 获取_工作表行数(self):
        return self.__sheet.max_row

    def 获取_工作表列数(self):
        return self.__sheet.max_column

    def 获取_单元格内容(self, 行: int, 列: int):
        '''#获取某一单元格内容：cell = sheet.cell(1,2).value、sheet['单元格'].value例如sheet['B1'].value
print(sheet.cell(20,20).value)'''
        return self.__sheet.cell(行, 列).value

    def 写入_单元格内容(self, 行: int, 列: int, 值):
        '行和列从1开始'
        self.__sheet.cell(row=行, column=列, value=值)

    def 获取_当前表格所有内容(self):
        # 按行读取 工作表的内容
        for row in self.__sheet.rows:
            for cell in row:
                print(cell.value, end=' ')
            print('')

    def 填充_单元格颜色(self, 行: int, 列: int, 颜色):
        '# 1开始 注意：fill_type为填充类型，如果不写的话，则没有效果。 默认纯色填充 颜色代号 白=ffffff 红=ff0000 黄=ffff00 绿=33ff00 蓝=0000ff 黑=000000 紫=9900ff'
        if 颜色 == '白':
            ls = 'ffffff'
        elif 颜色 == '黑':
            ls = '000000'
        elif 颜色 == '红':
            ls = 'ff0000'
        elif 颜色 == '黄':
            ls = 'ffff00'
        elif 颜色 == '绿':
            ls = '33ff00'
        elif 颜色 == '蓝':
            ls = '0000ff'
        else:
            ls = 'ffffff'
            print('输入颜色错误 恢复默认-白色')
        fill = openpyxl.styles.PatternFill(fill_type='solid', fgColor=ls)
        self.__sheet.cell(row=行, column=列).fill = fill

    def 保存_工作簿(self, 文件名):
        self.__wb.save(文件名)

    def 删除_当前工作表(self, 工作表名=None):
        '两种方法删除 一种是默认删除当前 一种可以选择'
        try:
            if 工作表名 == None:
                self.__wb.remove(self.__sheet)
            else:
                del self.__wb[工作表名]
        except IndexError:
            print('最少保留一个表格对象')

    def 退出(self):
        self.__wb.close()



import wx
import wx.lib.buttons
from .wxControl import *
import pyefun as efun


class 图形按钮(wx.lib.buttons.ThemedGenBitmapTextButton, 公用方法):
    pass

    _正常图片 = ""  # 储存图片路径以便在改变属性时刷新显示
    _焦点图片 = ""  # 储存图片路径以便在改变属性时刷新显示
    _禁止图片 = ""  # 储存图片路径以便在改变属性时刷新显示
    _按下图片 = ""  # 储存图片路径以便在改变属性时刷新显示
    _显示方式 = "图片居左上"  # 图片居左上 缩放图片 图片居中
    _图片缩放大小 = (0, 0)  # 宽度,高度

    def _处理图片的显示方式(self, value):
        if False == efun.文件是否存在(value):
            raise Exception("文件不存在 {}".format(value))
        if type(value) == str:
            value = wx.Image(value)
            if self._显示方式 == "图片居左上":
                pass
            elif self._显示方式 == "缩放图片":
                pass
                w, h = self._图片缩放大小[0],self._图片缩放大小[1]
                if w == 0 or h == 0:
                    w, h = self.GetSize()
                value = value.Scale(w, h)

            elif self._显示方式 == "图片居中":
                pass

            value = value.ConvertToBitmap()
        return value

    @property
    def 正常图片(self):
        return ""

    @正常图片.setter
    @组件_异常检测
    def 正常图片(self, value):
        if value == "":
            return
        图片 = self._处理图片的显示方式(value)
        self.SetBitmapLabel(图片)
        self._正常图片 = value

    @property
    def 显示方式(self):
        return self._显示方式

    @显示方式.setter
    def 显示方式(self, value):
        self._显示方式 = value

    @property
    def 图片缩放大小(self):
        return self._图片缩放大小

    @图片缩放大小.setter
    def 图片缩放大小(self, value):
        """ (w,h)"""
        self._图片缩放大小 = value
        self.正常图片 = self._正常图片
        self.焦点图片 = self._焦点图片
        self.按下图片 = self._按下图片
        self.禁止图片 = self._禁止图片

    @property
    def 焦点图片(self):
        return ""

    @焦点图片.setter
    @组件_异常检测
    def 焦点图片(self, value):
        if value == "":
            return
        图片 = self._处理图片的显示方式(value)
        self.SetBitmapFocus(图片)
        self._焦点图片 = value

    @property
    def 按下图片(self):
        return ""

    @按下图片.setter
    @组件_异常检测
    def 按下图片(self, value):
        if value == "":
            return
        图片 = self._处理图片的显示方式(value)
        self.SetBitmapSelected(图片)
        self._按下图片 = value

    @property
    def 禁止图片(self):
        return ""

    @禁止图片.setter
    @组件_异常检测
    def 禁止图片(self, value):
        if value == "":
            return
        图片 = self._处理图片的显示方式(value)
        self.SetBitmapDisabled(图片)
        self._禁止图片 = value

    def 取禁用状态图片(self):
        return self.GetBitmapDisabled()

    def 取焦点状态图片(self):
        return self.GetBitmapFocus()

    def 取正常状态图片(self):
        return self.GetBitmapLabel()

    def 取按下状态图片(self):
        return self.GetBitmapSelected()

    @组件_异常检测
    def 置禁用状态图片(self, 图片):
        return self.SetBitmapDisabled(图片)

    @组件_异常检测
    def 置焦点状态图片(self, 图片):
        return self.SetBitmapFocus(图片)

    @组件_异常检测
    def 置正常状态图片(self, 图片):
        return self.SetBitmapLabel(图片)

    @组件_异常检测
    def 置按下状态图片(self, 图片):
        return self.SetBitmapSelected(图片)



from selenium import webdriver
from selenium.common.exceptions import WebDriverException
from selenium.webdriver.common.desired_capabilities import DesiredCapabilities
import requests
from pyefun import *
import logging
import hashlib

logger = logging.getLogger()


def 取md5(内容):
    m = hashlib.md5(内容.encode("utf8"))
    return m.hexdigest()


def 浏览器_获取远程chrome(server_url="http://127.0.0.1:4444/wd/hub"):
    capabilities = DesiredCapabilities.CHROME.copy()
    capabilities['goog:chromeOptions'] = {
        "args": [
            # "--headless",
            # "window-size=1366,768"
            "user-agent=Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.77 Safari/537.36"
        ]
    }
    driver = webdriver.Remote(
        command_executor=server_url,
        desired_capabilities=capabilities
    )
    return driver


def 浏览器_获取本地chrome():
    opt = webdriver.ChromeOptions()
    # opt.add_argument('--headless')
    opt.add_argument('--disable-gpu')
    # opt.add_argument("blink-settings=imagesEnabled=false")
    opt.add_argument(
        "user-agent='Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.77 Safari/537.36'")
    # opt.add_argument('--proxy-server=http://%s' % "127.0.0.1:11111")
    driver = webdriver.Chrome(options=opt)

    return driver


def 浏览器_获取本地Firefox():
    opt = webdriver.FirefoxOptions()
    opt.add_argument('--headless')
    opt.add_argument('--disable-gpu')
    opt.add_argument("blink-settings=imagesEnabled=false")
    opt.add_argument(
        "user-agent='Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/62.0.3202.94 Safari/537.36'")
    driver = webdriver.Firefox(options=opt)
    return driver


def 浏览器_是否就绪(远程浏览器地址="http://127.0.0.1:4444/wd/hub"):
    """
        while 浏览器_是否就绪(远程浏览器地址) == False:
            延时(1)
            print("浏览器未就绪")
    """
    try:
        data = requests.get(远程浏览器地址 + "/status")
        json = data.json()
        ready = json['value']['ready']
    except:
        return False
    return ready


def 浏览器窗口最大化(driver):
    try:
        driver.maximize_window()
    except WebDriverException as e:
        driver.set_window_size(1920, 1080)  # 如果最大化失败，设置窗口大小为 1920*1080


def get_path_exists(path):
    for v in path:
        if os.path.exists(v):
            return v
    return ''


chromeDriverPath = ""
chromeBinaryLocation = ""


def 浏览器_函数计算环境初始化():
    if 系统_是否为window系统():
        return False
    global chromeDriverPath
    global chromeBinaryLocation
    if (chromeDriverPath != ""):
        return "浏览器环境已加载"
    # chromeDriverPath = "/opt/chrome86/chromedriver"
    # chromeBinaryLocation = "/opt/chrome86/headless-chromium"
    # data = 运行(chromeDriverPath+" --version") # 调试驱动是否正常
    # logger.info(data)
    # return data
    # return os.listdir("/opt/") # 查看文件是否部署到位

    chromeDriverPath = get_path_exists([
        "./chromedriver",
        "/opt/chromedriver",
        "./chrome86/chromedriver",
        "/opt/chrome86/chromedriver",
        "./chrome69/chromedriver",
        "/opt/chrome69/chromedriver",
    ])
    chromeBinaryLocation = get_path_exists([
        "./headless-chromium",
        "/opt/headless-chromium",
        "./chrome86/headless-chromium",
        "/opt/chrome86/headless-chromium",
        "./chrome69/headless-chromium",
        "/opt/chrome69/headless-chromium",
    ])
    # print("chromeDriverPath:%s" % chromeDriverPath)
    # print("chromeBinaryLocation:%s " % chromeBinaryLocation)
    if chromeDriverPath == "" or chromeBinaryLocation == "":
        exit("浏览器或驱动不存在")

    if os.access(chromeDriverPath, os.X_OK) == False or os.access(chromeBinaryLocation, os.X_OK) == False:
        # 如果没有执行权限那么复制到/tmp目录赋予执行权限
        chromeDriverPathTmp = "/tmp/chromedriver"
        chromeBinaryLocationTmp = "/tmp/headless-chromium"
        shutil.copyfile(chromeDriverPath, chromeDriverPathTmp)
        shutil.copyfile(chromeBinaryLocation, chromeBinaryLocationTmp)
        chromeDriverPath = chromeDriverPathTmp
        chromeBinaryLocation = chromeBinaryLocationTmp
        os.chmod(chromeDriverPath, 0o0700)
        os.chmod(chromeBinaryLocation, 0o0700)
    return "浏览器环境初始化完成"


def 输出(str):
    if 系统_是否为window系统():
        print(str)
    else:
        logger.info(str)


def 浏览器_自动获取chrome():
    global chromeDriverPath
    global chromeBinaryLocation
    if 系统_是否为window系统():
        return 浏览器_获取本地chrome()
    """
    用方便地将chrome 部署到部署到云上 方便 selenium的使用

    阿里云函数计算 腾讯云函数 linux docker中均可直接运行部署

    """
    chrome_options = webdriver.ChromeOptions()
    chrome_options.add_argument('--headless')
    chrome_options.add_argument('--no-sandbox')
    chrome_options.add_argument('--disable-gpu')
    chrome_options.add_argument('--window-size=1366x768')
    # chrome_options.add_argument('--user-data-dir=/tmp/user-data')
    chrome_options.add_argument('--hide-scrollbars')
    # chrome_options.add_argument('--enable-logging')
    # chrome_options.add_argument('--log-level=0')
    chrome_options.add_argument('--single-process')
    # chrome_options.add_argument('--data-path=/tmp/data-path')
    chrome_options.add_argument('--ignore-certificate-errors')
    chrome_options.add_argument('blink-settings=imagesEnabled=false')
    # chrome_options.add_argument('--homedir=/tmp')
    # chrome_options.add_argument('--disk-cache-dir=/tmp/cache-dir')
    chrome_options.add_argument(
        'user-agent=Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/90.0.4430.212 Safari/537.36')

    chrome_options.binary_location = chromeBinaryLocation

    chrome = webdriver.Chrome(chromeDriverPath, options=chrome_options)
    return chrome


if __name__ == '__main__':
    pass
    # 远程浏览器地址 = "http://127.0.0.1:4444/wd/hub"
    # while 浏览器_是否就绪(远程浏览器地址) == False:
    #     延时(1)
    #     print("浏览器未就绪")
    #
    # driver = 浏览器_获取远程chrome(远程浏览器地址)

    driver = 浏览器_获取本地chrome()
    data = driver.get("https://www.baidu.com")
    # print(data)
    # print(driver.find_element_by_xpath("//html").text)
    print(driver.find_elements_by_xpath("//html")[0].text)
    # print(driver.title)
    # print(driver.name)
    driver.quit()



"""
用于飞浆的分类模型 onnx 推理

需要安装 onnxruntime

"""

import time
import cv2
import numpy as np
import onnxruntime
from pyefun import *


def 输入图像预处理(输入数据):
    # 将输入数据转换为浮动32输入
    图像数据 = 输入数据.astype('float32')
    # 图像均值
    图像均值 = np.array([0.485, 0.456, 0.406])
    # 图像方差
    图像方差 = np.array([0.229, 0.224, 0.225])

    # 获得图像的矩阵用0填充
    标准图像数据 = np.zeros(图像数据.shape).astype('float32')
    # 图像归一化
    for i in range(图像数据.shape[0]):
        标准图像数据[i, :, :] = (图像数据[i, :, :] / 255 - 图像均值[i]) / 图像方差[i]

    # 添加输入通道
    标准图像数据 = 标准图像数据.reshape(1, 3, 224, 224).astype('float32')
    return 标准图像数据


def softmax(x):
    x = x.reshape(-1)
    e_x = np.exp(x - np.max(x))
    return e_x / e_x.sum(axis=0)


def 加载标签数据(标签文件路径):
    return 读入文本(标签文件路径).rstrip('\n').split('\n')


class 分类模型:
    def __init__(self, 模型文件路径, 标签文件路径=""):
        try:
            self.标签数据 = 读入文本(标签文件路径).rstrip('\n').split('\n')
        except:
            self.标签数据 = []
        self.模型 = onnxruntime.InferenceSession(模型文件路径)

    def 预测(self, 图片路径):
        图像 = cv2.imread(图片路径)
        # bgr 数据 转换为 rgb数据
        图像 = cv2.cvtColor(图像, cv2.COLOR_BGR2RGB)
        # 重置图像大小
        图像 = cv2.resize(图像, (224, 224))  # shape (224, 224, 3)  对应轴0 1 2
        图像数据 = np.array(图像).transpose(2, 0, 1)  # shape (3, 224, 224) 对应轴 2 1 0
        输入数据图像 = 输入图像预处理(图像数据)
        输出数据 = self.模型.run(output_names=None, input_feed={'image': 输入数据图像})
        self.result = softmax(np.array(输出数据)).tolist()
        idx = np.argmax(self.result)
        try:
            标签文本 = self.标签数据[idx]
        except:
            pass
            标签文本 = ""
        return idx, 标签文本

    def 取top1(self):
        return np.argmax(self.result)

    def 取top5(self):
        """前5预测结果"""
        sort_idx = np.flip(np.argsort(self.result))
        # print(分类标签[sort_idx[:5]])
        lable = []
        for k in sort_idx[:5]:
            try:
                lable.append(self.标签数据[k])
            except:
                pass
        return sort_idx[:5], lable




import json
import fc2

class 阿里云函数计算(fc2.Client):
    def __init__(self, endpoint, accessKeyID, accessKeySecret, Timeout=60):
        super(阿里云函数计算, self).__init__(
            endpoint=endpoint,
            accessKeyID=accessKeyID,
            accessKeySecret=accessKeySecret,
            Timeout=Timeout)

    def 调用事件函数(self, 服务名称, 函数名称, 参数=None, headers={}, qualifier=None, 异步调用=False):
        """

        参数 = bytes(json.dumps({
            "key": "hello",
            "url": "88888",
        }).encode("utf-8"))

        如果需要异步调用增加参数
        headers = {'x-fc-invocation-type': 'Async'}

        :param 服务名称:
        :param 函数名称:
        :param 参数:
        :param headers:
        :param qualifier:
        :return:

        返回参数

ret.data 字节集 函数的返回结果
ret.headers 头信息

{
	'Access-Control-Expose-Headers': 'Date,x-fc-request-id,x-fc-error-type,x-fc-code-checksum,x-fc-invocation-duration,x-fc-max-memory-usage,x-fc-log-result,x-fc-invocation-code-version',
	'Content-Length': '5',
	'Content-Type': 'application/octet-stream',
	'X-Fc-Code-Checksum': '校验数',
	'X-Fc-Invocation-Duration': '3',
	'X-Fc-Invocation-Service-Version': 'LATEST',
	'X-Fc-Max-Memory-Usage': '10.83',
	'X-Fc-Request-Id': '函数请求的id',
	'Date': 'Thu, 10 Jun 2021 18:46:23 GMT'
}
        """
        if 异步调用:
            headers["x-fc-invocation-type"] = 'Async'

        return self.invoke_function(服务名称, 函数名称, 参数, headers=headers, qualifier=qualifier)

    def 列出服务(self, limit=None, nextToken=None, prefix=None, startKey=None, headers={}, tags=None):
        return self.list_services(limit=limit, nextToken=nextToken, prefix=prefix, startKey=startKey, headers=headers,
                                  tags=tags)

    def 列出函数(self, serviceName, limit=None, nextToken=None, prefix=None, startKey=None, headers={}, qualifier=None):
        return self.list_functions(serviceName=serviceName, limit=limit, nextToken=nextToken, prefix=prefix,
                                   startKey=startKey, headers=headers, qualifier=qualifier)

    def 调用http函数(self, 方法名称, 服务名称, 函数名称, path, headers={}, params=None, body=None):
        """

        :param 方法名称: GET POST
        :param 服务名称:
        :param 函数名称:
        :param path:
        :param headers:
        :param params:
        :param body:
        :return:
        """
        return self.do_http_request(方法名称, 服务名称, 函数名称, path, headers=headers, params=params, body=body)




"""

.. Hint::
    正则表达式


.. literalinclude:: ../../../pyefun/正则表达式实用函数_test.py
    :language: python
    :caption: 代码示例
    :linenos:

"""
import re


# 正则匹配数字
def 正则匹配数字(数字):
    return re.match(r'^[0-9]*$', 数字).string


# 正则表达式匹配英文
def 正则匹配英文(英文):
    pattern = re.compile(r'[a-zA-Z]')
    return pattern.match(英文).string


# 正则表达式匹配手机号码
def 正则匹配中文(中文):
    pattern = re.compile(r'[\u4e00-\u9fa5]+')
    return pattern.match(中文).string


# 正则表达式匹配手机号码
def 正则匹配手机号码(手机号):
    """
    正则表达式匹配手机号码
    :param 手机号:
    :return:
    """
    pattern = re.compile(r'^1[3-9]\d{9}$')
    return pattern.match(手机号).string


# 正则表达式匹配邮箱
def 正则匹配邮箱(邮箱):
    """
    正则表达式匹配邮箱
    :param 邮箱:
    :return:
    """
    pattern = re.compile(r'^[a-zA-Z0-9_-]+@[a-zA-Z0-9_-]+(\.[a-zA-Z0-9_-]+)+$')
    return pattern.match(邮箱).string


# 正则表达式匹配身份证号码
def 正则匹配身份证号码(身份证号码):
    """
    正则表达式匹配身份证号码
    :param 身份证号码:
    :return:
    """
    pattern = re.compile(r'^[1-9]\d{5}[1-9]\d{3}((0\d)|(1[0-2]))(([0|1|2]\d)|3[0-1])\d{3}([0-9]|X)$')
    return pattern.match(身份证号码).string


# 正则表达式匹配银行卡号
def 正则匹配银行卡号(银行卡号):
    """
    正则表达式匹配银行卡号
    :param 银行卡号:
    :return:
    """
    pattern = re.compile(r'^[1-9]\d{9,22}$')
    return pattern.match(银行卡号).string


# 正则表达式匹配邮政编码
def 正则匹配邮政编码(邮政编码):
    """
    正则表达式匹配邮政编码
    :param 邮政编码:
    :return:
    """
    pattern = re.compile(r'^[1-9]\d{5}$')
    return pattern.match(邮政编码).string


# 正则表达式匹配IP地址
def 正则匹配IP地址(IP地址):
    """
    正则表达式匹配IP地址
    :param IP地址:
    :return:
    """
    pattern = re.compile(r'^((2[0-4]\d|25[0-5]|[01]?\d\d?)\.){3}(2[0-4]\d|25[0-5]|[01]?\d\d?)$')
    return pattern.match(IP地址).string


# 正则表达式匹配URL
def 正则匹配URL(URL):
    """
    正则表达式匹配URL
    :param URL:
    :return:
    """
    pattern = re.compile(r'^[a-zA-z]+://[^\s]*$')
    return pattern.match(URL).string


# 正则表达式匹配用户名
def 正则匹配用户名(用户名):
    """
    正则表达式匹配用户名
    :param 用户名:
    :return:
    """
    pattern = re.compile(r'^[a-zA-Z0-9_]{3,20}$')
    return pattern.match(用户名).string


# 正则表达式匹配密码
def 正则匹配密码(密码):
    """
    正则表达式匹配密码
    :param 密码:
    :return:
    """
    pattern = re.compile(r'^[a-zA-Z0-9_]{6,20}$')
    return pattern.match(密码).string



"""
.. Hint:: 公用模块
    异常处理等等


.. code-block:: python
   :linenos:

   @异常处理返回类型逻辑型
   def 如果函数发生错误即可提示兵忽略:
        print("没问题")
        a = 0/0 # 这个一定报错


"""
import re, sys, traceback, datetime

异常显示信息 = 2

报错_列表索引 = "取列表内容时，索引超出范围"
报错_层级 = "调用层级如下"
报错_除零 = "请勿除以零"
报错_递归 = "递归过深。请确认: 1、的确需要递归 2、递归的收敛正确"
报错_按索引取项 = "不支持按索引取项"
报错_无属性 = "没有属性"
参考_enter = "\n参考：https://stackoverflow.com/questions/1984325/explaining-pythons-enter-and-exit"

def 设置_异常处理_显示信息(显示信息=2):
    """

    :param 显示信息: 0 不显示 1显示简单的 2显示详细的
    :return:
    """
    global 异常显示信息
    异常显示信息 = 显示信息

class 层信息:
    def __init__(self, 行号, 内容, 文件名):
        self.行号, self.内容, self.文件名 = 行号, 内容, 文件名

def 提取(各层):
    各行 = []
    for 层号 in range(len(各层) - 1, -1, -1):
        层 = 各层[层号]
        文件名 = 层.filename
        各行.append(层信息(层.lineno, 层.line, 文件名))

    return 各行

def 报错信息(例外):
    exc_type, exc_value, 回溯信息 = sys.exc_info()
    各层 = traceback.extract_tb(回溯信息)
    # print(repr(各层))
    各行 = []

    行信息 = 提取(各层)
    类型 = 例外.__class__.__name__
    原信息 = str(例外)
    if len(行信息) > 0:
        关键 = 提示(类型, 原信息)
    各行.append(关键)
    for 行号, 行 in enumerate(行信息, start=1):
        # 在第二层前显示
        if 行号 == 2:
            各行.append(报错_层级)
        各行.append((f"见 File \"{行.文件名}\", ") + f"line {行.行号}：{行.内容}")
    return 各行

def 提示(类型, 原信息):
    if 类型 == 'NameError':
        return re.sub(r"name '(.*)' is not defined", r"请先定义‘\1’再使用", 原信息)
    elif 类型 == 'ZeroDivisionError':
        return 报错_除零
    elif 类型 == 'RecursionError':
        return 报错_递归
    elif 类型 == 'UnboundLocalError':
        return re.sub(
            r"local variable '(.*)' referenced before assignment",
            r"请先对本地变量‘\1’赋值再引用",
            原信息)
    elif 类型 == 'KeyError':
        return "字典中不存在此键：" + 原信息
    elif 类型 == 'TypeError':
        模式 = 'can only concatenate str \(not "(.*)"\) to str'
        无法取项 = "'(.*)' object is not subscriptable"
        参数缺失 = "(.*) missing 1 required positional argument: '(.*)'"
        if re.match(模式, 原信息):
            return re.sub(模式, r'字符串只能拼接字符串，请将“\1”先用 str() 转换', 原信息)
        匹配 = re.search(无法取项, 原信息)
        if 匹配:
            return f'{类型中文化(匹配.group(1))}{报错_按索引取项}'
        if re.match(参数缺失, 原信息):
            return re.sub(参数缺失, r'函数“\1”需要“\2”参数', 原信息)
    elif 类型 == 'IndexError' and 原信息 == "list index out of range":
        return 报错_列表索引
    elif 类型 == 'AttributeError':
        信息 = "需要添加此属性：" + 原信息
        if 原信息 == "__enter__":
            信息 += 参考_enter
            return 信息

        无属性 = "'(.*)' object has no attribute '(.*)'"
        匹配 = re.search(无属性, 原信息)
        if 匹配:
            return f'{类型中文化(匹配.group(1))}{报错_无属性}‘{匹配.group(2)}’'
    elif 类型 == 'FileNotFoundError':
        return re.sub(r"\[Errno 2\] No such file or directory: '(.*)'",
            r"没找到文件或路径：‘\1’",
            原信息)
    elif 类型 == 'ModuleNotFoundError':
        return re.sub(r"No module named '(.*)'", r"没找到模块：‘\1’", 原信息)
    return 类型 + "：" + 原信息

def 类型中文化(类型):
    中英表 = {
        "NoneType": "空变量",
        "int": "整数变量",
        "bool": "真假变量",
        "function": "函数",
    }
    return 中英表[类型] if 类型 in 中英表 else 类型

def 异常处理返回类型逻辑型(function):
    """
    使用方法 在def上面加上 @异常处理返回类型逻辑型

    例如

.. code-block:: python
   :linenos:

    @异常处理返回类型逻辑型
    def 如果函数发生错误即可提示兵忽略:
        print("没问题")
        a = 0/0 # 这个一定报错

    """

    def box(*args, **kwargs):
        try:
            return function(*args, **kwargs)
        except Exception as 例外:
            if 异常显示信息 == 2:
                print("函数异常 %s 时间 %s \r\n%s" % (
                    function.__name__,
                    str(datetime.datetime.now()),
                    "\n".join(报错信息(例外))
                ))
                # 调试用：对比原英文信息
                # print("\n\n" + traceback.format_exc())
            if 异常显示信息 == 1:
                print("函数异常 %s 时间 %s" % (
                    function.__name__,
                    str(datetime.datetime.now()),
                ))

            return False

    return box



import unittest

import pyautogui
from pyefun.模块.自动化模块 import *


class Test自动化操作(unittest.TestCase):
    def test_1(self):
        auto = 自动化模块()
        鼠标位置 = auto.获取当前鼠标位置()
        print(鼠标位置)
        鼠标位置 = auto.截图(r"C:\pyefun\pyefun\a\1.png", (0, 0, 300, 300))
        图片位置 = auto.找图(r"C:\pyefun\pyefun\a\1.png")
        print(图片位置)
        图片中心点位置 = auto.获取中心点位置(图片位置)
        print(图片中心点位置)



import wx.html2
import wx
from .wxControl import *

组件名称 = "浏览器"
组件创建代码 = """wx.浏览器(self.容器, size=({宽度}, {高度}), pos=({左边}, {顶边}))"""
界面设计器使用占位符创建 = True

class 浏览器(公用方法):
    pass

    def __init__(self, *args, **kw):
        self.浏览器对象 = wx.html2.WebView.New(*args, **kw)
        # self.__class__ = self.浏览器对象

    @property
    def 左边(self):
        return int(self.浏览器对象.GetPosition()[0])

    @左边.setter
    def 左边(self, value):
        return self.浏览器对象.Move(value, self.浏览器对象.GetPosition()[1])

    @property
    def 顶边(self):
        return int(self.浏览器对象.GetPosition()[1])

    @顶边.setter
    def 顶边(self, value):
        return self.浏览器对象.Move(self.浏览器对象.GetPosition()[0], value)

    @property
    def 宽度(self):
        return self.浏览器对象.GetSize()[0]

    @宽度.setter
    def 宽度(self, value):
        return self.浏览器对象.SetInitialSize((value, self.浏览器对象.GetSize()[1]))

    @property
    def 高度(self):
        return self.浏览器对象.GetSize()[1]

    @高度.setter
    def 高度(self, value):
        return self.浏览器对象.SetInitialSize((self.浏览器对象.GetSize()[0], value))

    def GetRect(self):
        return self.浏览器对象.GetRect()

    def GetId(self):
        return self.浏览器对象.GetId()

    def GetFont(self):
        return self.浏览器对象.GetFont()

    def IsShown(self):
        return self.浏览器对象.IsShown()

    def GetForegroundColour(self):
        return self.浏览器对象.GetForegroundColour()
    def GetLabel(self):
        return self.浏览器对象.GetLabel()

    def Raise(self):
        return self.浏览器对象.Raise()


    def GetBackgroundColour(self):
        return self.浏览器对象.GetBackgroundColour()

    def 绑定事件(self, *args, **kwargs):
        return self.浏览器对象.Bind(*args, **kwargs)

    def Bind(self, *args, **kwargs):
        return self.浏览器对象.Bind(*args, **kwargs)

    def Refresh(self, *args, **kwargs):
        return self.浏览器对象.Refresh(*args, **kwargs)

    def Move(self, *args, **kwargs):
        return self.浏览器对象.Move(*args, **kwargs)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time : 2021-06-12 23:33
# @Author : Nuonuo
# @Site : 
# @File : ffmpeg.py
# @Software: PyCharm

'''
ffmeg下载地址
http://www.ffmpeg.org/download.html
'''

import ffmpy3
import datetime
from moviepy.editor import *
import os


def 获取_视频_信息(path):
    data = 运行2(r'ffprobe -i ' + path)
    # Stream #0:0(und): Video: h264 (Main) (avc1 / 0x31637661), yuv420p, 960x540, 1008 kb/s, 25 fps, 25 tbr, 25k tbn, 50 tbc (default)
    # 第一个流是视频流，编码格式是H264格式(封装格式为AVC1)，每一帧的数据表示为yuv420p，分辨率为960x540，这路流的比特率为1108Kbit/s，帧率为每秒钟25帧。

    # Stream #0:1(und): Audio: aac (LC) (mp4a / 0x6134706D), 44100 Hz, stereo, fltp, 92 kb/s (default)
    # 这行信息表示第二个流是音频流，编码方式为ACC（封装格式为MP4A），并且采用的Profile是LC规格，采样率是44.1KHz，声道是立体声，这路流的比特率92Kbit/s。
    print(data)
    print('*' * 20)
    print('Duration: 持续时间,开始时间,比特率')
    print('Stream  编码格式,每一帧的数据表示,分辨率,这路流的比特率,帧率为每秒钟*帧')
    print('视频:Video 音频:Audio')


def 视频_压缩(path, 音频比特率, 视频比特率, 保存地址):
    '''
    改变码率  比特率
    ffmpeg -i 01.mp4 -b:a 100k -b:v 3000k nnn.mp4
    ffmpeg -i Desktop/1.mov -b:v 1.5M  Desktop/1.mp4
    改变分辨率
    ffmpeg -i Desktop/1.mov -s vga Desktop/1.mp4
    -s vga : 指定分辨率， vga 代表 600*480，也可以换成其他的值
    指定文件大小(不建议使用)
    ffmpeg -i Desktop/吉他.mp4  -fs 15MB  Desktop/output1.mp4
    #fs 20 : 表示文件大小最大值为15MB
    改变帕率
    ffmpeg -i Desktop/吉他.mp4  -r 20  Desktop/output1.mp4
    -r 20：表示帧率设置为 20fps
    ----------------------------
    一般 根据原来视频控制 比率
    '''
    ff = ffmpy3.FFmpeg(
        inputs={path: None},
        outputs={保存地址: f'-b:a {音频比特率}k -b:v {视频比特率}k'}
    )
    # print(ff.cmd)
    ff.run()


def 视频_切片_ts(path, 保存m3u8地址):
    # ffmpeg -i XXX.mp4 -c:v libx264 -c:a copy -f hls XXX.m3u8
    ff = ffmpy3.FFmpeg(
        inputs={path: None},
        outputs={保存m3u8地址: f' -c:v libx264 -c:a copy -f hls'}
    )
    # print(ff.cmd)
    ff.run()


def 视频_截取(path, 截取时间, 结束时间, 保存地址):
    '''
    path:截取视频文件
    截取时间:从第几分钟开始截取 1分钟1:00
    截取时长:需要截取多长时间
    保存地址:保存地址
    ---
    ffmpeg -ss 00:03:00 -i video.mp4 -to 00:02:00 -c copy cut.mp4
    去除片头 ffmpeg -ss 00:03:00 -i video.mp4 -c copy cut.mp4
    '''
    截取时长 = minNums(截取时间, 结束时间)
    ff = ffmpy3.FFmpeg(
        inputs={path: None},
        outputs={保存地址: f' -ss {截取时间} -t {截取时长} '}
    )
    # print(ff.cmd)
    ff.run()


def 视频_截取_快速(path, 截取时间, 截取时长, 保存地址):
    # 不推荐使用  文件大 嗯嗯嗯 视频不太稳  快是快
    ff = ffmpy3.FFmpeg(
        inputs={path: '-ss 截取时间'},
        outputs={保存地址: '-t 截取时长 -c:v copy -c:a copy'}
    )
    # print(ff.cmd)
    ff.run()


def 视频_提取音频(path, 保存地址):
    '音频默认m4a格式'
    # ffmpeg -i 16.mp4 -vn -codec copy out.m4a
    ff = ffmpy3.FFmpeg(
        inputs={path: None},
        outputs={保存地址: ' -vn -codec copy '}
    )
    # print(ff.cmd)
    ff.run()


def 视频_删除音频():
    pass
    '''
    去掉原视频音轨
    E:\anzhuangbao\ffmpeg\bin\ffmpeg -i G:\hi.mp4 -c:v copy -an G:\nosound.mp4
    添加背景音乐
    E:\anzhuangbao\ffmpeg\bin\ffmpeg -i G:\nosound.mp4 -i G:\songs.mp3 -t 7.1 -c y copy G:\output.mp4
    方法2
    合并音频和视频，保留视频原声（此时需要将mp3文件放在前面，MP4文件放在后面）否则会没有背景音乐
    E:\anzhuangbao\ffmpeg\bin\ffmpeg.exe -i G:\songs.mp3 -i G:\hi.mp4  -t 7.1 -y G:\new1.mp4
    -t后面跟时长 -y表示覆盖
    :return:
    '''


def minNums(startTime, endTime):
    '''计算两个时间点之间的分钟数'''
    # 处理格式,加上秒位
    startTime1 = startTime
    endTime1 = endTime
    # 计算分钟数
    startTime2 = datetime.datetime.strptime(startTime1, "%M:%S")
    endTime2 = datetime.datetime.strptime(endTime1, "%M:%S")
    seconds = (endTime2 - startTime2).seconds
    # 来获取时间差中的秒数。注意，seconds获得的秒只是时间差中的小时、分钟和秒部分的和，并没有包含时间差的天数（既是两个时间点不是同一天，失效）
    total_seconds = (endTime2 - startTime2).total_seconds()
    # 来获取准确的时间差，并将时间差转换为秒
    mins = total_seconds / 60
    s = total_seconds % 60

    if mins < 10:
        resu = '0' + str(int(mins))
    else:
        resu = '' + str(int(mins))
    if s < 10:
        resu += ':0' + str(int(s))
    else:
        resu += ':' + str(int(s))
    return resu


def 音频_MP3转WAV(path, 保存地址):
    # ffmpeg -i music.mp3 music.wav
    ff = ffmpy3.FFmpeg(
        inputs={path: None},
        outputs={保存地址: None}
    )
    # print(ff.cmd)
    ff.run()


def 音频_截取(path, 开始时间, 截取时长, 保存地址):
    # ffmpeg -i music.wav -ss 0 -t 37 musicshort.wav
    ff = ffmpy3.FFmpeg(
        inputs={path: None},
        outputs={保存地址: f' -ss {开始时间} -t {截取时长} '}
    )
    ff.run()


def 视频_音频_混合(音频地址, 视频地址, 保存地址):
    # ffmpeg -i musicshort.wav -i movie.avi final_video.avi
    ff = ffmpy3.FFmpeg(
        inputs={音频地址: None},
        outputs={保存地址: f'-i {视频地址} '}
    )
    ff.run()


def 视频_合成(filelist, 保存文件名):
    # file '10.mp4' 先生成txt文件
    # 操作目录默认桌面
    系统.修改当前操作路径(系统.取桌面目录())
    # ffmpeg -f concat -i filelist.txt -c copy out.mp4
    ff = ffmpy3.FFmpeg(
        inputs={None: "-f concat -i"},
        outputs={保存文件名: f' {filelist} -c copy '}
    )
    ff.run()


def 视频_合成多个(合成文件夹, 保存地址):
    # 注意某些视频可能因为分辨率问题影响视频合成

    # 主要是需要moviepy这个库
    # 定义一个数组
    L = []
    # 访问 video 文件夹 (假设视频都放在这里面)
    for root, dirs, files in os.walk(合成文件夹):
        # 遍历所有文件
        # 按文件名排序
        rq = []
        for i in files:
            rq.append(int(i[:-4]))
        rq.sort()
        for file in rq:
            # 如果后缀名为 .mp4
            # if os.path.splitext(file)[1] == '.mp4':
            # 拼接成完整路径
            filePath = os.path.join(root, str(file) + '.mp4')
            # 载入视频
            video = VideoFileClip(filePath)
            # 添加到数组
            L.append(video)
    # 拼接视频
    final_clip = concatenate_videoclips(L)
    # 生成目标视频文件
    final_clip.to_videofile(r"C:\Users\Erin\Desktop\\" + 保存地址, fps=24, remove_temp=False)


def 视频_合成单个(视频1, 视频2, 保存地址):
    系统.修改当前操作路径(系统.取桌面目录())
    # 修改视频帕高度和宽度 后合并
    video = VideoFileClip(视频1)
    video2 = VideoFileClip(视频2).resize(video.size)
    video_new = concatenate_videoclips([video2, video])
    video_new.write_videofile(r"C:\Users\Erin\Desktop\\" + 保存地址)


def 运行2(cmd):
    '运行cmd命令 并返回结果'
    return os.popen(cmd).read()



import base64

import cv2
import numpy as np

from PIL import Image, ImageDraw, ImageFont


class 图像操作类(object):
    """
    本类用于图像操作 使用cv2封装的类
    """

    def __init__(self, obj=None):
        """
        图像操作类初始化

        :param obj 可以是图像路径，图像字节集，图像base64
        """
        # print(type(obj))
        if type(obj) == str:
            self.img = cv2.imread(obj)
        else:
            self.img = obj

    # 创建空白图片
    def 创建空白图片(self, 宽度, 高度, 颜色=(255, 255, 255)):
        self.img = np.zeros((宽度, 高度, 3), np.uint8)
        self.img[:] = [颜色[2], 颜色[1], 颜色[0]]
        return self

    def 加载图片从文件(self, img_path):
        # 从文件中加载cv2对象
        data = np.fromfile(img_path, dtype=np.uint8)
        self.img = cv2.imdecode(data, cv2.IMREAD_ANYCOLOR)
        return self

    def 加载图片从字节集(self, img_bytes):
        # 从字节集中加载cv2对象
        data = np.frombuffer(img_bytes, np.uint8)
        self.img = cv2.imdecode(data, cv2.IMREAD_ANYCOLOR)
        return self

    def 加载图片从base64(self, img_bytes):
        img_bytes = base64.b64decode(img_bytes)
        img_array = np.frombuffer(img_bytes, np.uint8)
        self.img = cv2.imdecode(img_array, cv2.IMREAD_COLOR)
        return self

    def 加载图片从PIL(self, image):
        self.img = cv2.cvtColor(np.asarray(image), cv2.COLOR_RGB2BGR)
        return self

    # 图片转换为base64
    def 到base64(self):
        _, img_encode = cv2.imencode('.jpg', self.img)
        base64_str = base64.b64encode(img_encode.tobytes())
        return base64_str

    def 到PIL(self):
        return Image.fromarray(cv2.cvtColor(self.img, cv2.COLOR_BGR2RGB))

    @property
    def 宽度(self):
        return self.img.shape[1]

    @property
    def 高度(self):
        return self.img.shape[0]

    # 图片缩放
    def 缩放(self, scale=0.5):
        """
        图片操作.缩放(scale).显示图片()   scale为缩放比例 0.5为缩小一倍，2为放大一倍
        :param scale: 缩放比例
        :return:
        """
        self.img = cv2.resize(self.img, (int(self.img.shape[1] * scale), int(self.img.shape[0] * scale)))
        return self

    def 图像翻转左右(self):
        # flipCode 参数为-1.0,1,2 分别表示对角翻转,水平翻转，垂直翻转
        img = cv2.flip(self.img, 1)
        return 图像操作类(img)

    def 图像翻转上下(self):
        img = cv2.flip(self.img, 0)
        return 图像操作类(img)

    # 取某点颜色值
    def 取颜色(self, x, y):
        return self.img[y, x]

    # 绘制点
    def 绘制点(self, x, y, color=(0, 0, 255), 粗细=1):
        color = (color[2], color[1], color[0])
        cv2.circle(self.img, (x, y), 1, color, 粗细)
        return self

    def 剪裁(self, x1, y1, x2, y2):
        if (0 > x1):
            x1 = 0
        if (0 > y1):
            y1 = 0
        if (self.宽度 < x2):
            x2 = self.宽度
        if (self.高度 < y2):
            y2 = self.高度
        if x2 < x1:
            raise IndexError("尺寸不对")
        if y2 < y1:
            raise IndexError("尺寸不对")

        return 图像操作类(self.img[y1:y2, x1:x2])

    def 保存为文件(self, path=None):
        cv2.imwrite(path, self.img)
        return self

    def 到字节集(self):
        return cv2.imencode('.png', self.img)[1].tobytes()

    def 显示图片(self):
        """
        图片操作.显示图片()
        :return: -1为关闭了窗口 其他为键代码 可以使用 ord('1') 获取键代码
        """
        cv2.imshow("show", self.img)
        cv2.moveWindow("show" , 0, 0)
        on = cv2.waitKey(0)
        cv2.destroyAllWindows()
        return on

    # 图片二值化
    def 二值化(self, threshold=127):
        gray = cv2.cvtColor(self.img, cv2.COLOR_BGR2GRAY)
        ret, binary = cv2.threshold(gray, threshold, 255, cv2.THRESH_BINARY)
        return 图像操作类(binary)

    def 二值化自动(self):
        img = cv2.cvtColor(self.img, cv2.COLOR_BGR2GRAY);
        # # 二值化
        img = cv2.adaptiveThreshold(img, 255, cv2.ADAPTIVE_THRESH_MEAN_C, cv2.THRESH_BINARY, 27, 28)

        kernel = np.array([[0, -1, 0], [-1, 5, -1], [0, -1, 0]], np.float32)  # 锐化
        img = cv2.filter2D(img, -1, kernel=kernel)

        return 图像操作类(img)

    # 黑白颜色翻转
    def 颜色翻转(self):
        return 图像操作类(cv2.bitwise_not(self.img))

    def 绘制多边形(self, points, color=(255, 0, 0), 粗细=2):
        color = (color[2], color[1], color[0])
        cv2.polylines(self.img, [np.array(points, dtype=np.int32)], True, color, 粗细)
        return self

    def 绘制矩形(self, x1, y1, x2, y2, color=(255, 0, 0), thickness=1):
        color = (color[2], color[1], color[0])
        cv2.rectangle(self.img, (x1, y1), (x2, y2), color, thickness)
        return self

    def 绘制文本(self, text, x, y, font=cv2.FONT_HERSHEY_SIMPLEX, fontScale=0.5, color=(255, 0, 0), thickness=1):
        cv2.putText(self.img, text, (x, y), font, fontScale, color, thickness)
        return self

    def 绘制文本汉字(self, text, x, y, color=(255, 0, 0), font_size=30):
        img = self.到PIL()
        draw = ImageDraw.Draw(img)
        fontStyle = ImageFont.truetype("msyh.ttc", font_size, encoding="utf-8")
        draw.text((x, y), text, color, font=fontStyle)
        self.img = cv2.cvtColor(np.asarray(img), cv2.COLOR_RGB2BGR)
        return self

    def 绘制填充矩形(self, x1, y1, x2, y2, color=(255, 0, 0), transparency=0.5):
        blk = np.zeros(self.img.shape, np.uint8)
        cv2.rectangle(blk, (x1, y1), (x2, y2), color, -1)
        blk = cv2.cvtColor(blk, cv2.COLOR_RGB2BGR)
        self.img = cv2.addWeighted(self.img, 1.0, blk, transparency, 1)
        return self

    def 绘制填充多边形(self, points, color=(255, 0, 0), transparency=0.5):
        blk = np.zeros(self.img.shape, np.uint8)
        cv2.fillPoly(blk, [np.array(points, dtype=np.int32)], color)
        blk = cv2.cvtColor(blk, cv2.COLOR_RGB2BGR)
        self.img = cv2.addWeighted(self.img, 1.0, blk, transparency, 1)
        return self



import unittest

from .图像处理 import *
from .图像处理PIL import *
from .图像处理函数 import *


class TestCacheUtil(unittest.TestCase):

    def test_PIL(self):
        图片操作 = 图像操作PIL类().创建空白图片(100, 200, color=(255, 0, 0)).显示图片()
        # 图片操作 = 图像操作PIL类(r"9108efa6518a1fbadfa77047.jpeg")
        # 图片数据 = 图片操作.绘制点(10, 10).显示图片()
        # 图片数据 = 图片操作.绘制圆形(100, 100, 100).显示图片()
        # 图片数据 = 图片操作.绘制填充圆形(100, 100, 100).显示图片()
        # 图片数据 = 图片操作.剪裁(0, 0, 100, 100).保存为文件("test1.jpg").显示图片()
        # 图片数据 = 图片操作.剪裁(0, 0, 100, 100).二值化().显示图片()
        # 图片数据 = 图片操作.绘制矩形(0, 0, 100, 100).绘制文本汉字("123啊", 48, 48, (255, 0, 0)).显示图片()
        # 图片数据 = 图片操作.绘制填充矩形(0, 0, 100, 100, (255, 0, 0)).显示图片()
        # 图片数据 = 图片操作.绘制多边形([[0, 74], [200, 74], [300, 142], [0, 142]], (255, 0, 0)).显示图片()
        # 图片数据 = 图片操作.绘制填充多边形([[0, 74], [200, 74], [300, 142], [0, 142]], (255, 0, 0)).显示图片()
        # 图片数据 = 图片操作.二值化().显示图片()
        # 图片数据 = 图片操作.二值化()
        # print(图片数据.到base64())

        # 图像操作类2().加载图片从base64(图片数据.到base64()).显示图片()
        # 图片数据 = 图片操作.二值化(180)
        # 图片操作.二值化(127).颜色翻转().显示图片()
        # 图片操作.图像翻转左右().显示图片()
        # 图片操作.图像翻转上下().显示图片()
        # 图片操作.增强对比度().显示图片()
        # 图片操作.增强亮度().显示图片()
        # 图片操作.锐利度(0).显示图片()
        # 图片操作.高斯模糊(3).显示图片()
        #
        # 图片操作.边缘检测().显示图片()
        # 图片操作.轮廓().显示图片()

    def test_CV2(self):
        pass
        图片操作 = 图像操作类(r"9108efa6518a1fbadfa77047.jpeg")
        图片数据 = 图片操作.绘制点(10, 10).显示图片()
        # # 图片数据 = 图片操作.图像翻转上下().显示图片()
        # 图片数据 = 图片操作.图像翻转左右().显示图片()
        # print(图片数据)
        # 图像操作类().创建空白图片(100, 200, (255, 255, 255)).绘制文本汉字("哈哈哈", 0, 0).绘制点(50, 50, (255, 0, 0), 10).显示图片()

        # 图片数据 = 图片操作.剪裁(0, 0, 100, 100).保存为文件("test1.jpg").显示图片()
        # 图片数据 = 图片操作.剪裁(0, 0, 100, 100).二值化()
        # 图片数据 = 图片操作.绘制矩形(0, 0, 100, 100).绘制文本汉字("123啊", 48, 48, (255, 0, 0)).显示图片()
        # 图片数据 = 图片操作.绘制填充矩形(0, 0, 100, 100, (255, 0, 0)).显示图片()
        # 图片数据 = 图片操作.绘制多边形([[0, 74], [200, 74], [300, 142], [0, 142]], (255, 0, 0)).显示图片()
        # 图片数据 = 图片操作.绘制填充多边形([[0, 74], [200, 74], [300, 142], [0, 142]], (255, 0, 0)).显示图片()
        # 图片数据 = 图片操作.二值化().显示图片()
        # 图片数据 = 图片操作.二值化().转换为base64()
        # print(图片数据.到base64())

        # 图像操作类().加载图片从base64(图片数据.到base64()).显示图片()
        # 图片数据 = 图片操作.二值化(180)
        # 图片数据 = 图像操作类(图片数据).颜色翻转()
        # 图像操作类(图片数据).显示图片()

    def test_实用函数(self):
        def test_1(self):
            pass
            文件路径 = 路径_合并(取运行目录(), r"example\wxefun\2.png")
            # print(文件路径)
            字节集 = 读入文件(文件路径)
            print(len(字节集))

            im = image从字节集加载(字节集)
            # print(im)
            # im.show()
            字节集 = image到字节集(im)
            print(len(字节集))

            im = image从字节集加载(字节集)
            # print(im)
            # im.show()
            字节集 = image到字节集(im)
            print(len(字节集))

            print(image取图片宽度高度(im))

            # image显示图片(im)


"""

.. Hint::
    自动化模块 采用 pyautogui 模块进行封装 实现模拟窗口操作的功能 自动模拟人工操作
    需要安装 pip install pyautogui


.. literalinclude:: ../../../pyefun/自动化模块_test.py
    :language: python
    :caption: 代码示例
    :linenos:

"""

from pyefun import *
import pyautogui
import pyperclip
import ppppocr
import winapi
import tempfile


class 自动化模块():
    ocr_md5 = None

    def __init__(self):
        pass

    def 设置保护措施(self, 每一步延迟=2.5):
        # 当pyautogui.FAILSAFE = True时，如果把鼠标光标在屏幕左上角，PyAutoGUI函数就会产生pyautogui.FailSafeException异常。
        pyautogui.PAUSE = 每一步延迟
        pyautogui.FAILSAFE = True

    def 当前屏幕分辨率(self):
        return pyautogui.size()

    def 找图(self, 图片路径):
        #  返回 (left=0, top=0, width=300, height=400) #
        try:
            return pyautogui.locateOnScreen(图片路径)
        except:
            return False

    def 找图灰度模式(self, 图片路径):
        #  返回 (left=0, top=0, width=300, height=400) #
        try:
            return pyautogui.locateOnScreen(图片路径, grayscale=True)
        except:
            return False

    def 找图所有(self, 图片路径):
        #  返回 [Box(left=0, top=0, width=300, height=400)] #
        try:
            return pyautogui.locateAllOnScreen(图片路径)
        except:
            return False

    def 找图_中心点坐标(self, 图片路径):
        #  返回 (left=0, top=0, width=300, height=400) #
        try:
            return pyautogui.locateOnScreen(图片路径)
        except:
            return False

    def 获取中心点位置(self, 坐标):
        return pyautogui.center(坐标)

    def 获取当前鼠标位置(self):
        #  返回 (x, y) #
        return pyautogui.position()

    def 获取当前鼠标位置颜色(self):
        #  返回 (r, g, b) #
        return pyautogui.screenshot().getpixel(self.获取当前鼠标位置())

    def 截图全屏(self, 图片保存路径):
        pyautogui.screenshot(图片保存路径)  # 截全屏并设置保存图片的位置和名称

    def 检查指定位置的像素值(self, x, y, 颜色值=(255, 255, 245), 误差值=10):
        pyautogui.pixelMatchesColor(x, y, 颜色值, tolerance=误差值)

    def 截图(self, 图片保存路径, 截屏区域=(0, 0, 100, 100)):
        #  例如 auto.截图(截图地址, (left, top, width, height)) #
        pyautogui.screenshot(图片保存路径, region=截屏区域)

    def 鼠标移动(self, x, y, 时间=0):
        pyautogui.moveTo(x, y, duration=时间)

    def 鼠标拖动(self, x, y, 时间=0, 按键="left"):
        # 按键 设置为'left'、'middle'，以及'right'
        pyautogui.dragTo(x, y, 时间, button=按键)

    def 鼠标点击(self, x, y):
        pyautogui.click(x, y)

    def 鼠标右击(self, x, y):
        pyautogui.rightClick(x, y)

    def 鼠标双击(self, x, y):
        pyautogui.doubleClick(x, y)

    def 鼠标三击(self, x, y):
        pyautogui.tripleClick(x, y)

    def 鼠标左键按下(self, x, y):
        pyautogui.mouseDown(x, y)

    def 鼠标左键弹起(self, x, y):
        pyautogui.mouseUp(x, y)

    def 鼠标右键按下(self, x, y):
        pyautogui.rightClick(x, y)

    def 鼠标右键弹起(self, x, y):
        pyautogui.rightClick(x, y)

    def 鼠标滚轮向下(self, x, y):
        pyautogui.scroll(x, y)

    def 鼠标滚轮向上(self, x, y):
        pyautogui.scroll(-x, -y)

    def 鼠标滚轮向左(self, x, y):
        pyautogui.scroll(-x, -y)

    def 鼠标滚轮向右(self, x, y):
        pyautogui.scroll(x, y)

    def 键盘输入文字(self, 内容, 时间=0):
        # 例如 键盘输入文字('hello') #
        pyautogui.typewrite(内容, interval=时间)

    def 键盘按下(self, 键名):
        # 例如: 键盘按下('enter') #
        pyautogui.keyDown(键名)

    def 键盘弹起(self, 键名):
        # 例如 键盘弹起('shift') #
        pyautogui.keyUp(键名)

    def 键盘按下并弹起(self, 键名):
        # 例如 键盘按下并弹起('enter')#
        pyautogui.press(键名)

    def 键盘按下热键(self, *args, **kwargs):
        # 例如 键盘按下热键("ctrl", "c") #
        pyautogui.hotkey(*args, **kwargs)

    def 弹窗提示框(self, 内容, 标题='提示', 按钮='确定'):
        # 例如 弹窗提示('hello') #
        return pyautogui.alert(text=内容, title=标题, button=按钮)

    def 弹窗确认框(self, 内容, 标题='提示', 按钮=('确定', '取消')):
        # 例如 弹窗提示并等待('hello') #
        return pyautogui.confirm(内容, title=标题, buttons=按钮)

    def 弹窗输入框(self, 内容, 标题='提示'):
        return pyautogui.prompt(text=内容, title=标题, default='')

    def 弹出密码输入框(self, 内容, 标题):
        return pyautogui.password(text=内容, title=标题, default='', mask='*')

    def 获取窗口句柄(self, 窗口标题="微信"):
        return pyautogui.getWindowsWithTitle(窗口标题)

    def 粘贴(self, 内容):
        pyperclip.copy(内容)
        pyautogui.hotkey('ctrl', 'v')

    def 可视化工具(self):
        """
        MouseInfo的使用方法有两种。

        方法1：

        勾选3 Sec. Button Delay
        点击copy all(推荐)或者 copy xy 或者copy RGB
        然后在三秒内将鼠标移动到想要选择的位置
        等到3秒时候MouseInfo会记录下当前鼠标位置和颜色
        方法2：

        取消勾选3 Sec. Button Delay
        将鼠标移至想选择的位置
        然后按F1至F8键复制或记录鼠标位置，这样就把这些按键与鼠标记录做了关联映射
        可以MouseInfo窗口顶部的“Copy”和“Log”菜单，以找出按键映射到哪些按钮。
        """
        pyautogui.mouseInfo()

    def 启用ocr(self):
        self.ocr = ppppocr.ppppOcr()

    def ocr(self, 图片地址):
        self.dt_boxes, self.rec_res = self.ocr.ocr(图片地址)
        print(self.dt_boxes, self.rec_res)
        return self.dt_boxes, self.rec_res

    def 获取窗口找字区域(self, 窗口句柄):
        left, top, right, bottom = win32apiUtil.窗口_取窗口矩形(窗口句柄)
        width = right - left + 1
        height = bottom - top + 1
        # print(top, left, width, height)
        # win32apiUtil.窗口_置焦点(窗口句柄)
        return (left, top, width, height)

    def 找字(self, 字, 截图区域=None):
        截图保存地址 = os.path.join(tempfile.gettempdir(), "1.png")
        # print(截图保存地址)
        if 截图区域 == None:
            auto.截图全屏(截图保存地址)
        else:
            auto.截图(截图保存地址, 截图区域)

        # 图像没有变化的话 就继续使用缓存结果了
        _md5 = 取数据md5(读入文件(截图保存地址))
        if self.ocr_md5 != _md5:
            self.dt_boxes, self.rec_res = self.ocr.ocr(截图保存地址)
            self.ocr_md5 = _md5
            删除文件(截图保存地址)

        # print(self.dt_boxes, self.rec_res)
        print(self.rec_res)
        for i in range(len(self.dt_boxes)):
            txt = self.rec_res[i][0]
            x1, y1, x2, y2 = self.dt_boxes[i][0][0], self.dt_boxes[i][0][1], self.dt_boxes[i][2][0], \
                             self.dt_boxes[i][2][1]
            中心点x = x1 + 1
            中心点y = y1 + 1
            if 字 == txt:
                print("找到了,", 字, txt, x1, y1, x2, y2)
                return 中心点x, 中心点y
        return False

#
# if __name__ == '__main__':
#     auto = 自动化模块()
#     auto.启用ocr()
#     # x, y = auto.找字("回收站", (0, 0, 500, 500))
#     # auto.鼠标双击(x, y)
#     # 延时(2)
#     x, y = auto.找字("此电脑")
#     auto.鼠标双击(x, y)
#     延时(3)
#
#     x, y = auto.找字("照片")
#     auto.鼠标双击(x, y)
#     延时(3)
#
#     x, y = auto.找字("本机照片")
#     auto.鼠标双击(x, y)

# auto.可视化工具获取鼠标位置()

#     # 鼠标位置 = auto.获取当前鼠标位置()
#     # print(鼠标位置)
#     # 鼠标位置 = auto.获取当前鼠标位置颜色()
#     # print(鼠标位置)
#     # auto.鼠标移动(14, 20)
#     # 延时(1)
#     # auto.鼠标点击(14, 20)
#     鼠标位置 = auto.截图(r"C:\pyefun\pyefun\a\1.png", (0, 0, 300, 300))
#     图片位置 = auto.找图(r"C:\pyefun\pyefun\a\1.png")
#     print(图片位置)
#     图片中心点位置 = auto.获取中心点位置(图片位置)
#     print(图片中心点位置)



import unittest

import pyefun.ai.ONNXClassInference as onnx


class TestONNXInference(unittest.TestCase):

    def test_1(self):
        pass

        ONNX分类模型 = onnx.分类模型(
            模型文件路径="./狗狗品种分类模型/onnx/onnx_file",
            # 标签文件路径="./狗狗品种分类模型/onnx/labels.txt"
        )
        标签索引,标签文本 = ONNX分类模型.预测(图片路径="./Alaskan_malamute_00346.jpg")
        print(标签索引,标签文本)
        print(ONNX分类模型.取top1(),ONNX分类模型.取top5())




"""
.. Hint::
    网络操作


.. literalinclude:: ../../../pyefun/网络操作_test.py
    :language: python
    :caption: 代码示例
    :linenos:

"""
from pyefun import *


@异常处理返回类型逻辑型
def 网页_取外网IP():
    ehttp = eRequests()
    ehttp.关闭调试信息()

    返回文本 = ehttp.get("http://pv.sohu.com/cityjson")
    ip = strCut(返回文本.文本, '"cip": "$"')
    if ip != "":
        return ip
    返回文本 = ehttp.get("https://ipservice.ws.126.net/locate/api/getLocByIp?callback=bowlder.cb._2")
    ip = strCut(返回文本.文本, '"ip":"$"')
    if ip != "":
        return ip

    返回文本 = ehttp.get("https://api.bilibili.com/x/web-interface/zone?jsonp=jsonp")
    ip = strCut(返回文本.文本, '"addr":"$"')
    if ip != "":
        return ip

    return ""




import wx
from .wxControl import *

组件名称 = "选择夹"
组件创建代码 = """wx.选择夹(self.容器, size=({宽度}, {高度}), pos=({左边}, {顶边}), style=wx.NB_FIXEDWIDTH)"""
界面设计器使用占位符创建 = True

class 选择夹(wx.Notebook, 公用方法):
    pass



import unittest

from 剪切板操作 import *
from pyefun import *


class TestclipBoard(unittest.TestCase):

    def test_1(self):
        pass
        置剪辑板文本("易函数")
        self.assertEqual(取剪辑板文本(), "易函数")
        清除剪辑板()
        self.assertEqual(取剪辑板文本(), "")




# -*- coding: utf-8 -*-
# @Time : 2021-10-31 15:14
# @Author : LiGuangLong
import redis


class Redis:
    def __init__(self, localhost, port, db=0):
        """redis中默认16个数据库 默认使用第一个 每个库互相不干扰"""
        pool = redis.ConnectionPool(host=localhost, db=db, port=port, decode_responses=True)
        self.r = redis.Redis(connection_pool=pool)

    def 添加value(self, key, value, 过期时间秒=None, 过期时间毫秒=None, nx=None, xx=None):
        """
        在 Redis 中设置值，默认，不存在则创建，存在则修改
        nx - 如果设置为True，则只有name不存在时，当前set操作才执行
        xx - 如果设置为True，则只有name存在时，当前set操作才执行
        """
        return self.r.set(key, value, 过期时间秒, 过期时间毫秒, nx, xx)

    def 切片添加(self, key, 索引, value):
        return self.r.setrange(key, 索引, value)

    def 追加值(self, key, value):
        return self.r.append(key, value)

    def 自增value(self, key, 步长=1):
        """名称为key的string增1操作"""
        return self.r.incr(key, 步长)

    def 值增加(self, key, 增加值):
        """名称为key的string增加integer"""
        return self.r.incrby(key, 增加值)

    def 自减value(self, key, 步长=1):
        return self.r.decr(key, 步长)

    def 值减少(self, key, 减少值):
        return self.r.decrby(key, 减少值)

    def 获取value(self, key):
        return self.r.get(key)

    def 获取多个value(self, keys, *args):
        return self.r.mget(keys, *args)

    def 获取并修改(self, key, value):
        """设置新值并获取原来的值"""
        return self.r.getset(key, value)

    def 切片获取(self, key, start, end):
        return self.r.getrange(key, start, end)

    def key是否存在(self, key):
        """1存在 0 不存在"""
        return self.r.exists(key)

    def 获取key存在时间(self, key):
        return self.r.ttl(key)

    def 设置key存在时间(self, key, 时间):
        """单位秒"""
        return self.r.expire(key, 时间)

    def 移动key位置(self, key, dbindex数据库):
        return self.r.move(key, dbindex数据库)

    def 获取全部key(self):
        return self.r.keys()

    def 获取key数量(self):
        """返回当前数据库中key的数目"""
        return self.r.dbsize()

    def 删除(self, key):
        return self.r.delete(key)

    def 查看value类型(self, key):
        return self.r.type(key)

    def 清空当前库(self):
        return self.r.flushdb()

    def 清空全部数据(self):
        return self.r.flushall()

    def 切换数据库(self):
        return

    #####list操作 Redis的list类型其实就是一个每个子元素都是string类型的双向链表，链表的最大长度是2^32。list既可以用做栈，也可以用做队列。
    def list添加尾(self, key, *value):
        return self.r.rpush(key, *value)

    def list添加头(self, key, *value):
        return self.r.lpush(key, *value)

    def 获取list长度(self, key):
        return self.r.llen(key)

    def list获取(self, key, index=None, start=None, end=None):
        if index is not None:
            return self.r.lindex(key, index)
        elif start is not None and end is not None:
            return self.r.lrange(key, start, end)
        else:
            return self.r.lrange(key, 0, -1)

    def list下标赋值(self, key, index, value):
        """给名称为key的list中index位置的元素赋值"""
        return self.r.lset(key, index, value)

    def list删除多个value(self, key, 数量, value):
        """删除多个key的list中值为value的元素"""
        return self.r.lrem(key, 数量, value)

    def list删除首(self, key):
        """返回并删除名称为key的list中的首元素"""
        return self.r.lpop(key)

    def list删除尾(self, key):
        """：返回并删除名称为key的list中的尾元素"""
        return self.r.rpop(key)

    def list元素移动(self, srckey, dstkey):
        """返回并删除名称为srckey的list的尾元素，并将该元素添加到名称为dstkey的list的头部"""
        return self.r.rpoplpush(srckey, dstkey)

    ############################未封装完毕###############################32
    """
    Set
    特点：无序性、确定性、唯一性
    
    sadd(key, member)：向名称为key的set中添加元素member
    srem(key, member) ：删除名称为key的set中的元素member
    spop(key) ：随机返回并删除名称为key的set中一个元素
    smove(srckey, dstkey, member) ：移到集合元素
    scard(key) ：返回名称为key的set的基数
    sismember(key, member) ：member是否是名称为key的set的元素
    sinter(key1, key2,…key N) ：求交集
    sinterstore(dstkey, (keys)) ：求交集并将交集保存到dstkey的集合
    sunion(key1, (keys)) ：求并集
    sunionstore(dstkey, (keys)) ：求并集并将并集保存到dstkey的集合
    sdiff(key1, (keys)) ：求差集
    sdiffstore(dstkey, (keys)) ：求差集并将差集保存到dstkey的集合
    smembers(key) ：返回名称为key的set的所有元素
    srandmember(key) ：随机返回名称为key的set的一个元素
    Hash
    Redis hash 是一个string类型的field和value的映射表，它的添加、删除操作都是O(1)（平均）。hash特别适用于存储对象，将一个对象存储在hash类型中会占用更少的内存，并且可以方便的存取整个对象。
    
    hset(key, field, value)：向名称为key的hash中添加元素field
    hget(key, field)：返回名称为key的hash中field对应的value
    hmget(key, (fields))：返回名称为key的hash中field i对应的value
    hmset(key, (fields))：向名称为key的hash中添加元素field
    hincrby(key, field, integer)：将名称为key的hash中field的value增加integer
    hexists(key, field)：名称为key的hash中是否存在键为field的域
    hdel(key, field)：删除名称为key的hash中键为field的域
    hlen(key)：返回名称为key的hash中元素个数
    hkeys(key)：返回名称为key的hash中所有键
    hvals(key)：返回名称为key的hash中所有键对应的value
    hgetall(key)：返回名称为key的hash中所有的键（field）及其对应的value
    Zest
    概念：它是在set的基础上增加了一个顺序属性，这一属性在添加修改元素的时候可以指定，每次指定后，zset会自动按新的值调整顺序。可以理解为有两列的mysql表，一列存储value，一列存储顺序，操作中key理解为zset的名字。
    
    zadd key score1 value1：添加元素
    zrange key start stop [withscore]：把集合排序后,返回名次[start,stop]的元素 默认是升续排列 withscores 是把score也打印出来
    zrank key member：查询member的排名（升序0名开始）
    zrevrank key member：查询member排名（降序 0名开始）
    zrem key value1 value2：删除集合中的元素
    zremrangebyrank key start end：按排名删除元素，删除名次在[start, end]之间的
    zcard key：返回集合元素的个数
    zcount key min max：返回[min, max]区间内元素数量

    """


if __name__ == '__main__':
    r = Redis('121.43.149.217', 6379, 1)
    # print(r.清空全部数据())

    for i in r.获取全部key():
        print(f'{i}------{r.获取key存在时间(i)}')



"""

.. Hint::
    网络请求 eRequests 是实用的http请求模块
    轻松配置cookie文件保存 全局请求头信息 代理ip
    ehttp = eRequests(cookies文件路径="cookie文件路径",全局头信息="User-Agent: pyefun")
    ehttp.设置全局HTTP代理("127.0.0.1:11111")

    返回文本 = ehttp.get("http://127.0.0.1:9000/1.php")
    print(返回文本.文本)

    返回文本 = ehttp.post("http://127.0.0.1:9000/post.php", 发送文本={"aa": 1, "bb": 2, "cc": 3})
    print(返回文本.文本)

    # 文件上传
    字节集 = 读入文件("share.png")
    返回文本 = ehttp.post("http://127.0.0.1:9000/post.php", 发送文本={"aa": 1, "bb": 2, "cc": 3},
                      上传文件={'upload': ('code.png', 字节集, 'image/png')})

.. literalinclude:: ../../../pyefun/网络请求_test.py
    :language: python
    :caption: 代码示例
    :linenos:
    :lines: 1-100

"""
from pyefun import *
from pyefun.核心易函数支持库.时间统计 import *  # 时间工具类

from requests.adapters import HTTPAdapter
from http.cookiejar import LWPCookieJar
import requests
from requests import exceptions


class ehttp响应类(object):
    Response: requests.Response

    def __init__(self, obj):
        self.Response = obj

    def Response(self):
        return self.Response

    def json(self):
        return self.Response.json()

    @property
    def 内容(self):
        return self.Response.content

    @property
    def 文本(self):
        return self.Response.text

    @property
    def 字节集(self):
        return self.Response.content

    @property
    def 头信息(self):
        return self.Response.headers

    @property
    def 状态码(self):
        return self.Response.status_code

    @property
    def 编码(self):
        return self.Response.encoding

    @property
    def url(self):
        return self.Response.url

    @property
    def cookies(self):
        return self.Response.cookies

    @property
    def 取Location(self):
        return self.Response.headers['Location']

    @property
    def 取重定向URl(self):
        return self.Response.headers['Location']

    @property
    def 访问失败(self):
        # 真 访问失败
        # 假 访问成功
        if self.状态码 == 200 or self.状态码 == 302 or self.状态码 == 301:
            return False
        else:
            return True


from urllib.parse import urlparse


def 网址_取域名(url):
    parse_result = urlparse(url)
    return parse_result.netloc

def 屏蔽Requests中的警告信息():
    import warnings
    warnings.simplefilter('ignore', ResourceWarning)
    requests.packages.urllib3.disable_warnings()

class eRequests(object):
    req = requests.session
    cookies文件路径 = ""
    全局HTTP代理 = None
    调试信息 = False
    默认头信息 = """Accept : */*
User-Agent: Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/86.0.4240.198 Safari/537.36
Content-Type: application/x-www-form-urlencoded"""
    全局头信息 = ""

    def __init__(self, cookies文件路径="", 访问失败重试次数=0, 全局头信息=""):
        pass
        self.req = requests.session()
        self.调试信息 = False
        self.cookies文件路径 = cookies文件路径
        self.全局HTTP代理 = None
        self.req.mount('http://', HTTPAdapter(max_retries=访问失败重试次数))
        self.req.mount('https://', HTTPAdapter(max_retries=访问失败重试次数))
        self.设置自动管理cookies(cookies文件路径)
        if 全局头信息 != "":
            self.设置全局头信息(全局头信息)
        self.设置全局HTTP代理("")

    def __del__(self):
        if self.cookies文件路径 != "":
            self.req.cookies.save()

    def 设置自动管理cookies(self, 文件路径):
        pass
        self.cookies文件路径 = 文件路径
        self.req.cookies = LWPCookieJar(filename=文件路径)
        if 文件是否存在(文件路径):
            self.req.cookies.load()

    # 默认情况下对象销毁就会自动保存了 无需调用此方法
    def cookies保存到文件(self):
        if self.cookies文件路径 != "":
            self.req.cookies.save()

    def 设置全局HTTP代理(self, 代理ip):
        """格式 127.0.0.1:8080"""
        pass
        self.全局HTTP代理 = {
            'http': 代理ip,
            'https': 代理ip
        }

    def 设置全局头信息(self, 全局头信息):
        """头信息将全局添加到请求头中"""
        # print(type(全局头信息))
        if type(全局头信息) == dict:
            重新处理 = ""
            for k, v in 全局头信息.items():
                重新处理 = 重新处理 + "{k}: {v}\r\n".format(k=k, v=v)
            self.全局头信息 = 重新处理
        else:

            self.全局头信息 = 全局头信息

    def 关闭调试信息(self, 关闭调试信息=True):
        pass
        self.调试信息 = 关闭调试信息

    def get(self, url: str, 附加头信息: str = "", 允许重定向=True, 超时=15, 不使用代理访问=False):
        return self.访问(url=url, 访问方法="GET", 发送文本="", 附加头信息=附加头信息, 允许重定向=允许重定向, 超时=超时, 不使用代理访问=不使用代理访问)

    def delete(self, url: str, 附加头信息: str = "", 允许重定向=True, 超时=15, 不使用代理访问=False):
        return self.访问(url=url, 访问方法="GET", 发送文本="", 附加头信息=附加头信息, 允许重定向=允许重定向, 超时=超时, 不使用代理访问=不使用代理访问)

    def head(self, url: str, 附加头信息: str = "", 允许重定向=True, 超时=15, 不使用代理访问=False):
        return self.访问(url=url, 访问方法="GET", 发送文本="", 附加头信息=附加头信息, 允许重定向=允许重定向, 超时=超时, 不使用代理访问=不使用代理访问)

    def options(self, url: str, 附加头信息: str = "", 允许重定向=True, 超时=15, 不使用代理访问=False):
        return self.访问(url=url, 访问方法="GET", 发送文本="", 附加头信息=附加头信息, 允许重定向=允许重定向, 超时=超时, 不使用代理访问=不使用代理访问)

    def post(self, url: str, 发送文本: str = "", 附加头信息: str = "", 允许重定向=True, 超时=15, 不使用代理访问=False, 上传文件=None):
        return self.访问(url=url, 访问方法="POST", 发送文本=发送文本, 附加头信息=附加头信息, 允许重定向=允许重定向, 超时=超时, 不使用代理访问=不使用代理访问, 上传文件=上传文件)

    def 访问(self, url: str, 访问方法: str = "GET", 发送文本: str = "", 附加头信息: str = "", 允许重定向=True, 超时=15, 不使用代理访问=False,
           上传文件=None):
        if not self.调试信息:
            t = 时间统计()


        全局HTTP代理 = self.全局HTTP代理
        if 不使用代理访问 == True:
            全局HTTP代理 = {'http': "", 'https': ""}

        timeout = (超时, 超时)

        headers = {}

        # 兼容各种传入方式
        if type(附加头信息) == dict:
            重新处理 = ""
            for k, v in 附加头信息.items():
                重新处理 = 重新处理 + "{k}: {v}\r\n".format(k=k, v=v)
            _附加头信息 = 重新处理
        else:
            _附加头信息 = 附加头信息

        附加头信息 = self.默认头信息 + "\n" + self.全局头信息 + "\n" + _附加头信息

        for 文本内容 in 附加头信息.split('\n'):
            截取位置 = 寻找文本(文本内容, ":")
            名称 = 删首尾空(取文本左边(文本内容, 截取位置))
            值 = 删首尾空(取文本右边(文本内容, 取文本长度(文本内容) - 截取位置 - 1))
            if 名称 and 值:
                headers[名称] = 值

        附加cookies = {}
        cookies = headers.get('Cookie', "")
        if cookies.find(";") != -1:
            for 文本内容 in cookies.split(';'):
                文本内容 = 删首尾空(文本内容)
                等于号位置 = 寻找文本(文本内容, "=")
                名称 = 取文本左边(文本内容, 等于号位置)
                值 = 取文本右边(文本内容, 取文本长度(文本内容) - 等于号位置 - 1)
                if 名称 and 值:
                    附加cookies[名称] = 值

        # 加入cookie
        requests.utils.add_dict_to_cookiejar(self.req.cookies, 附加cookies)

        # 这里获取cookie貌似是所有的...查阅代码未解决 本想获取当前访问域名的cookies

        # print(self.req.cookies.get_dict(网址_取域名(url),"/"))

        if not self.调试信息:
            # 获取请求之前加载的cookie
            加载cookie = ""
            for k, v in requests.utils.dict_from_cookiejar(self.req.cookies).items():
                if k != "":
                    加载cookie = 加载cookie + "{k}={v}; ".format(k=k, v=v)
            if 加载cookie != "":
                加载cookie = "\r\n" + "加载 Cookies {cookies}".format(cookies=加载cookie)

        try:
            if 访问方法 == "POST":
                返回数据 = self.req.post(url=url, data=发送文本, proxies=全局HTTP代理, timeout=timeout, allow_redirects=允许重定向,
                                     verify=False,
                                     headers=headers, files=上传文件)
            if 访问方法 == "GET":
                返回数据 = self.req.get(url=url, proxies=全局HTTP代理, timeout=timeout, allow_redirects=允许重定向, verify=False,
                                    headers=headers)
            if 访问方法 == "delete":
                返回数据 = self.req.delete(url=url, proxies=全局HTTP代理, timeout=timeout, allow_redirects=允许重定向, verify=False,
                                       headers=headers)
            if 访问方法 == "head":
                返回数据 = self.req.head(url=url, proxies=全局HTTP代理, timeout=timeout, allow_redirects=允许重定向, verify=False,
                                     headers=headers)
            if 访问方法 == "options":
                返回数据 = self.req.options(url=url, proxies=全局HTTP代理, timeout=timeout, allow_redirects=允许重定向, verify=False,
                                        headers=headers)


        except Exception as e:
            # print('http请求错误:' + str(e))
            返回数据 = ehttp响应类(requests.Response())
            返回数据.status_code = 0
            if not self.调试信息:
                self._输出调试信息(访问方法, url, 加载cookie, 全局HTTP代理, t, 返回数据, "\r\n错误信息 " + str(e))
            return 返回数据

        if not self.调试信息:
            self._输出调试信息(访问方法, url, 加载cookie, 全局HTTP代理, t, 返回数据)

        return ehttp响应类(返回数据)

    def _输出调试信息(self, 访问方法, url, 加载cookie, 全局HTTP代理, t, 返回数据, 错误信息=""):
        if not self.调试信息:
            耗时 = t.取秒()
            # print(返回数据.headers.get('Set-Cookie'))
            cookies = ""
            for item in 返回数据.cookies.items():
                cookies = cookies + "{0}={1}; ".format(item[0], item[1])
            if cookies != "":
                cookies = "\r\n" + "新增 Cookies {cookies}".format(cookies=cookies)

            重定向地址 = ""
            if 返回数据.status_code == 301 or 返回数据.status_code == 302:
                重定向地址 = "重定向URL " + 返回数据.headers.get('Location')

            代理信息 = 全局HTTP代理.get('http', "")
            if 代理信息 != "":
                代理信息 = "代理 " + 代理信息
            状态码 = 返回数据.status_code

            print(
                "{访问方法} {url} 状态码 {状态码} 耗时 {耗时}s {代理信息} {重定向地址} {加载cookie} {cookies} {错误信息}".format(访问方法=访问方法, url=url,
                                                                                                    状态码=状态码,
                                                                                                    耗时=耗时,
                                                                                                    cookies=cookies,
                                                                                                    加载cookie=加载cookie,
                                                                                                    重定向地址=重定向地址,
                                                                                                    代理信息=代理信息,
                                                                                                    错误信息=错误信息))



import unittest

from .通用文字识别 import *


class Test通用文字识别(unittest.TestCase):
    def test_ocr(self):
        文字位置, 文字内容 = 通用文字识别(r"C:\pyefun\pyefun\a\1.png")
        # print(文字位置)
        print(文字内容)
        文字内容 = 通用文字识别获取文字(r"C:\pyefun\pyefun\a\1.png")
        print(文字内容)
        Json内容 = 通用文字识别获取Json(r"C:\pyefun\pyefun\a\1.png")
        print(Json内容)



"""

.. Hint::
    图像处理

    pip install pillow

.. literalinclude:: ../../../pyefun/imageUtil_test.py
    :language: python
    :caption: 代码示例
    :linenos:


"""
import os

from PIL import Image, ImageGrab
import io


def image到字节集(image对象):
    字节流 = io.BytesIO()
    image对象.save(字节流, format='PNG')
    图片字节集 = 字节流.getvalue()
    字节流.close()
    return 图片字节集


def image从字节集加载(图片字节集) -> Image:
    return Image.open(io.BytesIO(图片字节集))


def 图片_修改尺寸(图片路径, 保存地址, 横=100, 竖=100):
    im = Image.open(图片路径)
    dImg = im.resize((横, 竖), Image.ANTIALIAS)  # 横竖长度
    dImg.save(保存地址)


def 图片_压缩(infile, outfile='', mb=150, step=10, quality=80):
    """不改变图片尺寸压缩到指定大小
    :param infile: 压缩源文件
    :param outfile: 压缩文件保存地址
    :param mb: 压缩目标，KB
    :param step: 每次调整的压缩比率
    :param quality: 初始压缩比率
    :return: 压缩文件地址，压缩文件大小
    """
    o_size = os.path.getsize(infile) / 1024
    if o_size <= mb:
        return infile

    while o_size > mb:
        im = Image.open(infile)
        im.save(outfile, quality=quality)
        if quality - step < 0:
            break
        quality -= step
        o_size = os.path.getsize(outfile) / 1024
    return outfile, os.path.getsize(outfile) / 1024


def 截图_取当前屏幕(图片路径="picture.jpg"):
    """
    保存在当前目录下picture.jpg文件
    """
    pic = ImageGrab.grab()
    pic.save(图片路径)
